# -*- coding: utf-8 -*-

# @Time     : 2021/10/30 0:22
# @Software : PyCharm
# @License  : GNU General Public License v3.0
# @Author   : xxx
import pyximport
from mgetool.tool import tt

pyximport.install()
import magic
import magic2
a = magic.A()
b = magic2.A()

tt.t
a.foo(1000)
tt.t
b.foo(1000)
tt.t
tt.p


# -*- coding: utf-8 -*-

# @Time    : 2021/8/6 15:42
# @Email   : 986798607@qq.com
# @Software: PyCharm
# @License: BSD 3-Clause
import re

try:
    from translate import Translator
except ImportError:
    raise ImportError("'translate' should be install. Try ``pip install translate``")


def translate_en_to_zh(words: str):
    translator = Translator(to_lang="zh")
    return translator.translate(words)

def remove_empty(lists):
    return [i for i in lists if i != '']


def get_msgid(text_lines):
    text = "".join(text_lines)
    text_ms = text.split("msgid ")
    text_cup = [i.split("msgstr ") for i in text_ms]
    data = []

    for i in text_cup:
        datai=[]
        if len(i)==2:
            str_msgid = remove_empty(i[0].split("\n"))
            str_msgstr = remove_empty(i[1].split("\n"))
            datai.append(str_msgid)
            datai.append(str_msgstr)
        else:
            str_ = remove_empty(i[0].split("\n"))
            datai.append(str_)
        data.append(datai)

    data_new = []
    for data_cup in data:
        if len(data_cup) == 2:
            for i in range(len(data_cup[0])):
                try:
                    new = translate_en_to_zh(data_cup[0][i])
                    data_cup[1][i]=new
                except BaseException:
                    pass
            data_new.append(data_cup)

    return data

# f = open(r"C:\Users\Administrator\PycharmProjects\featurebox\docs\locale\zh_CN\LC_MESSAGES\src\featurebox.selection.po")
# text_ = f.readlines()
# s= get_msgid(text_lines=text_)






#!/usr/bin/python3.7
# -*- coding: utf-8 -*-

# @Time   : 2019/7/29 19:47
# @Author : Administrator
# @Software: PyCharm
# @License: BSD 3-Clause


"""
Notes:
    import data simply.
    # Just a copy from xenonpy
    for pictures use skimage.io.ImageCollection

"""
import copy
import glob
import os
import re
import shutil
from collections import defaultdict
from functools import partial
from pathlib import Path
from warnings import warn

import joblib
import pandas as pd
import requests
from skimage import io
from tqdm import tqdm

from mgetool.tool import def_pwd, parallelize


class Call(object):
    """
    Call file in paths.

    When there are four files in pwd path:
        (file1.csv, file2.csv, file3.txt, file4.png)

    Examples
    ---------
    >>> call = Call(".",backend="csv")
    >>> file1 = call.file1
    >>> file2 = call.file2
    >>> call = Call(".",backend="txt")
    >>> file = call.file3

    """

    @staticmethod
    def extension(index_col=0):
        read_csv = partial(pd.read_csv, index_col=index_col)
        read_excel = partial(pd.read_excel, index_col=index_col)
        extension = dict(
            pkl_pd=('pkl_pd', pd.read_pickle),
            csv=('csv', read_csv),
            xlsx=('xlsx', read_excel),
            pkl_sk=('pkl_sk', joblib.load),
            png=("png", io.imread),
            jpg=("jpg", io.imread),
        )
        return extension

    __re__ = re.compile(r'[\s\-.]')

    def __init__(self, *paths, backend='pkl_pd', prefix_with_upper=None, index_col=0):
        """

        Parameters
        ----------
        paths:list
            list of path
        backend:str
            default imported type to show
        prefix_with_upper:str
            prefix_with_upper for all file add to file in this code to escape same name
        index_col:
            use the first column as index in table.
        """

        self._backend = backend
        self.index_col = index_col
        self._files = None
        self.__extension__ = self.extension(index_col)

        if len(paths) == 0:
            self._paths = ('.',)
        else:
            self._paths = paths

        if not prefix_with_upper:
            prefix_with_upper = ()
        self._prefix = prefix_with_upper

        self._make_index(prefix_with_upper=prefix_with_upper)

    def _make_index(self, *, prefix_with_upper):
        def make(path_):
            patten = self.__extension__[self._backend][0]
            files = glob.glob(str(path_ / ('*.' + patten)))

            def _nest(_f):
                f_ = _f
                return lambda s: s.__extension__[s._backend][1](f_)

            for f in files:
                # selection data_cluster
                f = Path(f).resolve()
                parent = re.split(r'[\\/]', str(f.parent))[-1]
                # parent = str(f.parent).split('\\/')[-1]
                fn = f.name[:-(1 + len(patten))]
                fn = self.__re__.sub('_', fn)
                if prefix_with_upper:
                    fn = '_'.join([parent, fn])

                if fn in self._files:
                    warn("file %s with x_name %s already bind to %s and will be ignored" %
                         (str(f), fn, self._files[fn]), RuntimeWarning)
                else:
                    self._files[fn] = str(f)
                    setattr(self.__class__, fn, property(_nest(str(f))))

        self._files = defaultdict(str)
        for path in self._paths:
            path = Path(path).expanduser().absolute()
            if not path.exists():
                raise RuntimeError('%s not exists' % str(path))
            make(path)

    @classmethod
    def from_http(cls, url, save_to, *, filename=None, chunk_size=256 * 1024, params=None,
                  **kwargs):
        """
        Get file object via a http request.

        Parameters
        ----------
        url: str
            The resource url.
        save_to: str
            The path of a path to save the downloaded object into it.
        filename: str, optional
            Specific the file x_name when saving.
            Set to ``None`` (default) to use a inferred x_name from http header.
        chunk_size: int, optional
            Chunk size.
        params: any, optional
            Parameters will be passed to ``requests.get`` function.
            See Also: `requests <http://docs.python-requests.org/>`_
        kwargs: dict, optional
            Pass to ``requests.get`` function as the ``kwargs`` parameters.

        Returns
        -------
        str
            File path contains file x_name.
        """
        r = requests.get(url, params, **kwargs)
        r.raise_for_status()

        if not filename:
            if 'filename' in r.headers:
                filename = r.headers['filename']
            else:
                filename = url.split('/')[-1]

        if isinstance(save_to, str):
            save_to = Path(save_to)
        if not isinstance(save_to, Path) or not save_to.is_dir():
            raise RuntimeError('%s is not a legal path or not point to a path' % save_to)

        file_ = str(save_to / filename)
        with open(file_, 'wb') as f:
            for chunk in r.iter_content(chunk_size=chunk_size):
                if chunk:  # filter out keep-alive new chunks
                    f.write(chunk)

        return file_

    def __repr__(self):
        cont_ls = ['<{}> includes:'.format(self.__class__.__name__)]

        for k, v in self._files.items():
            cont_ls.append('"{}": {}'.format(k, v))

        return '\n'.join(cont_ls)

    def csv(self):
        return Call(*self._paths, backend='csv', prefix_with_upper=self._prefix, index_col=self.index_col)

    def pickle_pd(self):
        return Call(*self._paths, backend='pkl_pd', prefix_with_upper=self._prefix, index_col=self.index_col)

    def pickle_sk(self):
        return Call(*self._paths, backend='pkl_sk', prefix_with_upper=self._prefix, index_col=self.index_col)

    def xlsx(self):
        return Call(*self._paths, backend='xlsx', prefix_with_upper=self._prefix, index_col=self.index_col)

    def png(self):
        return Call(*self._paths, backend='png', prefix_with_upper=self._prefix, index_col=self.index_col)

    def jpg(self):
        return Call(*self._paths, backend='jpg', prefix_with_upper=self._prefix, index_col=self.index_col)

    def __call__(self, *args, **kwargs):
        return self.__extension__[self._backend][1](*args, **kwargs)

    def __getattr__(self, name):
        """
        Returns sub-dataset.

        Parameters
        ----------
        name: str
            Dataset x_name.

        Returns
        -------
        spath
        """
        if name in self.__extension__:
            return self.__class__(*self._paths, backend=name, prefix_with_upper=self._prefix)
        else:
            raise AttributeError("'%s' object has no attribute '%s'" % (self.__class__.__name__, name))


def check_file(spath, file_path, suffix=None):
    os.chdir(file_path)
    # print(os.path.abspath(os.curdir))
    all_file = os.listdir('.')
    files = []
    for f in all_file:
        if os.path.isdir(f):
            ff = os.path.join(file_path, f)
            files.extend(check_file(spath, ff, suffix=suffix))
            os.chdir(file_path)
        else:
            if not suffix:
                che = True
            elif suffix == "":
                che = "" == os.path.splitext(f)[1]
            else:
                che = "".join((".", suffix)) == os.path.splitext(f)[1]
            if che:
                rel_path = file_path.replace(spath, "")
                parents = re.split(r'[\\/]', str(rel_path))
                files.append([parents, f])
            else:
                pass
    return files


class BatchFile:
    r"""
    Search files and filter files and re-site files.

    Examples
    ---------
    >>> a = BatchFile(".")
    >>> a.filter_dir_name("a")
    >>> a.filter_file_name("2")
    >>> print(a.file_list)
    ...
    #copy the file to new path and keep the dir structure
    >>> a.to_path(r"C:\Users\Admin\Desktop\d2",flatten=False)
    #copy the file to new path, flatten the file and add the dir name on file: dirname_1_filename.
    >>> a.to_path(r"C:\Users\Admin\Desktop\d2", add_dir=[-1], flatten=True)
    #copy the file to new path, flatten the file and add the dir name on file: dirname_2_dirname_1_filename.
    >>> a.to_path(r"C:\Users\Admin\Desktop\d2", add_dir=[-2,-1], flatten=True)

    """

    def __init__(self, path=None, suffix=None):
        """

        Parameters
        ----------
        path:str
            total dir of all file
        suffix:str
            suffix of file
            Examples:
                .txt
        """

        path = def_pwd(path)
        self.path = path

        parents = re.split(r'[\\/]', str(path))
        self.parents = parents

        self.file_list = check_file(path, path, suffix=suffix)
        self.init_file = tuple(self.file_list)
        self.file_list_merge = []
        self.file_list_merge_new = []
        self.file_dir = []

    def filter_file_name(self, include=None, exclude=None):
        """

        Parameters
        ----------
        include:str
            get the filename with include str
            such as hold "ast_tep" with "ast" string
        exclude: str
            delete the filename with exclude str
            such as hold "ast_cap" and delete "ast_tep" with "tep" str,

        """

        if include is None and exclude is None:
            return

        assert include != []
        assert exclude != []
        if isinstance(include, str):
            include = [include, ]
        if isinstance(exclude, str):
            exclude = [exclude, ]

        file_list_filter = []
        for file_i in self.file_list:
            name = file_i[1]

            if include and not exclude:
                if any([i in name for i in include]):
                    file_list_filter.append(file_i)
            elif not include and exclude:
                if not any([i in name for i in exclude]):
                    file_list_filter.append(file_i)
            elif include and exclude:
                if any([i in name for i in include]) and not any([i in name for i in exclude]):
                    file_list_filter.append(file_i)
            else:
                raise TypeError("one of include, exclude must be str or list of str")

        self.file_list = file_list_filter

    def filter_dir_name(self, include=None, exclude=None, layer=-1):
        """
        Filter the dir(and its sub_file).

        Parameters
        ----------
        include:str,list of str
            get the filename with include str
            such as hold "ast_tep" with "ast" string
        exclude: str, list of str
            delete the filename with exclude str
            such as hold "ast_cap" and delete "ast_tep" with "tep" str,
        layer:int,list
            if list, check the sum name of the layers.
            Filter dir with target layer,all the dir should contain the sublayer!
            Examples:
                for /home,
                /home/ast, -1
                /home/ast/eag, -2
                /home/ast/eag/kgg, -3

        """
        if include is None and exclude is None:
            return

        assert include != []
        assert exclude != []
        if isinstance(include, str):
            include = [include, ]
        if isinstance(exclude, str):
            exclude = [exclude, ]

        file_list_filter = []

        for file_i in self.file_list:
            try:
                if isinstance(layer, int):
                    layer = [layer, ]

                if isinstance(layer, list):
                    name = [file_i[0][i] for i in layer]
                else:
                    name = file_i[0]
                name = "".join(name)

                if include and not exclude:
                    if any([i in name for i in include]):
                        file_list_filter.append(file_i)
                elif not include and exclude:
                    if not any([i in name for i in exclude]):
                        file_list_filter.append(file_i)
                elif include and exclude:
                    if any([i in name for i in include]) and not any([i in name for i in exclude]):
                        file_list_filter.append(file_i)
                else:
                    raise TypeError("one of include, exclude must be str or list of str")
            except IndexError:
                pass

        self.file_list = file_list_filter

    def merge(self, path=None, flatten=False, add_dir="3-layer", refresh_file_list=True, pop=0):
        """
        Merge dir and file name together.

        Parameters
        ----------
        path:str
            new path
        flatten:True
            flatten the filtered file.
            if flatten is dict, the key is the specific dir name,and value is True.
            Examples:
            flatten = {"asp":True}
        add_dir:int,list
            add the top dir_name to file to escape same name file.
            only valid for flatten=True
        refresh_file_list:bool
            refresh file_list or not.
        pop: int (negative)
            pop the last n layer. default =0
            used for copy by dir rather than files. just used for flatten=False

        Returns
        -------
            new filename

            Args:
                refresh_file_list:
                refresh_file_list:
        """
        if not path:
            path = self.path
            flatten = False
        if not add_dir:
            add_dir = []
        elif add_dir == "3-layer":
            add_dir = [-1, -2, -3]
        if isinstance(add_dir, int):
            add_dir = [add_dir, ]

        if flatten is not False:
            assert pop == 0
        assert pop <= 0

        file_list_merge = []
        for file_i in self.file_list:
            site = copy.copy(file_i[0])
            if isinstance(flatten, dict):
                site = [site[_] for _ in add_dir]
                site_c = ""
                for i, j in enumerate(site):
                    i -= len(site)
                    if i in flatten.keys():
                        if flatten[i] in [True, "layer", "dir", "folder", 1, "s"]:
                            site_c += "".join((j, "/"))
                        else:
                            site_c += "".join((j, "_"))
                    else:
                        site_c += "".join((j, "_"))
                site_c = re.split(r'[\\/]', str(site_c))
                site_c[-1] += file_i[1]
                file_list_merge.append(os.path.join(path, *site_c))

            elif flatten:
                site = [site[_] for _ in add_dir]
                site.append(file_i[1])
                site = "_".join(site)
                file_list_merge.append(os.path.join(path, site))
            else:

                site.append(file_i[1])
                if pop != 0:
                    site = site[:pop]
                namei = os.path.join(path, *site)

                if len(file_list_merge) == 0 or namei != file_list_merge[-1]:
                    file_list_merge.append(namei)

        if refresh_file_list:
            self.file_list_merge = file_list_merge
        fdir = list(set([os.path.dirname(i) for i in file_list_merge]))
        fdir.sort()
        self.file_dir = fdir
        return file_list_merge

    def to_path(self, new_path, flatten=False, add_dir="3-layer", pop=0, n_jobs=1):
        """

        Parameters
        ----------
        new_path:str
            new path
        flatten:bool,dict
            flatten the filtered file.
            if flatten is dict, the key is the specific dir name,and value is True.
            Examples:
            flatten = {"asp":True}
        add_dir:list, int
            add the top dir_name to file to escape same name file.
            only valid for flatten=True
        pop: int (negative)
            pop the last n layer. default =0
            used for copy by dir rather than files. just used for flatten=False
        n_jobs:int
            n_jobs

        Returns
        -------
            file in path.
        """
        self.file_list_merge = self.merge(pop=pop)
        new_path = def_pwd(new_path)
        self.file_list_merge_new = self.merge(path=new_path, flatten=flatten, add_dir=add_dir,
                                              refresh_file_list=False, pop=pop)
        if len(set(self.file_list_merge_new)) < len(set(self.file_list_merge)):
            raise UserWarning("There are same name files after flatten folders. "
                              "you can change add_dir to add difference prefix to files", )
        if n_jobs != 1:
            parallelize(n_jobs, self.copy_user, zip(self.file_list_merge, self.file_list_merge_new, ),
                        mode="j",
                        respective=False)
        else:
            for ij in tqdm(list(zip(self.file_list_merge, self.file_list_merge_new))):
                self.copy_user(ij)

    @staticmethod
    def copy_user(k):
        i, j = k
        if os.path.isdir(i):
            shutil.copytree(i, j)
        else:
            path_i = os.path.split(j)[0]
            if not os.path.exists(path_i):
                os.makedirs(path_i)
            shutil.copy(i, j)

# if __name__ == "__main__":
# others please use shutil
# shutil.copytree()
# a = BatchFile(r"C:\Users\wangchangxin\Desktop\d1")
# a.filter_dir_name("a", layer=-1)
# a.filter_file_name("2")
# a.to_path(r"C:\Users\wangchangxin\Desktop\d2", add_dir=[-2, -1], flatten=True)
# bf = BatchFile(r"/home/iap13/wcx/CHG")

# bf.filter_dir_name(include="Mo")
# filenames = bf.file_list



"""
Import dynamic library.
"""

import imp
import importlib.util
import os
import platform
import shutil

from mgetool.tool import def_pwd


def import_module(module_name, path):
    """import module from path"""
    file, path, description = imp.find_module(module_name, [path])
    # Close the .so file after load.
    with file:
        return imp.load_module(module_name, file, path, description)


def import_source(module_name, log_print=False):
    """import module from module.__init__.py path."""
    module_file_path = module_name.__file__
    module_name = module_name.__name__

    module_spec = importlib.util.spec_from_file_location(module_name, module_file_path)
    module = importlib.util.module_from_spec(module_spec)
    module_spec.loader.exec_module(module)
    if log_print:
        print(dir(module))
        msg = "The {module_name} module has the following methods:{methods}"
        print(msg.format(module_name=module_name, methods=dir(module)))


def re_build_func(module_name=None, with_html=False):
    """
    Just run build: 'python setup.py build_ext --inplace'.

    Parameters
    ----------
    module_name: str
        just for pyx file.
    with_html:bool
        just for pyx file.

    """
    if os.path.isdir("build"):
        shutil.rmtree("build")
    print("############### build >>> ###############".format(module_name))
    val = os.system("python setup.py build_ext --inplace")
    if with_html:  # just for pyx
        print("build {}.html".format(module_name))
        os.system("cython -a {}.pyx".format(module_name))
    if val != 0:
        raise FileNotFoundError("run 'python setup.py build_ext --inplace' failed due to no 'setup.py' \n"
                                "or error in 'setup.py'. \n"
                                "If without setup.py, please turn to draft to generate 'setup.py' \n")
    else:
        print("########### build end ###############".format(module_name))


def quick_import(module_name, path=None, build=False, suffix=".so", with_html=False, re_build_func=re_build_func,
                 re_build_func_kwargs=None, log_print=False
                 ):
    """
    Import .so file as module.

    Parameters
    ----------
    re_build_func_kwargs:dict
        kwargs for build func
    re_build_func:callable
        build func
    module_name: str
        module_name
    path:
        path
    build:
        build or not.
        default is false and try to find the exist module_name.so or module_name.pyd or module_name.dll.
    suffix:str
        file type [so,pyd] and so on
    with_html:False
        just for cython to check file.

    Returns
    -------
    module

    """
    if suffix is None:
        sys_name = platform.system()
        suffix = ".so" if sys_name == "Linux" else ".pyd"

    def_pwd(path)
    if log_print:
        print("Move to {}".format(os.getcwd()))

    if build:
        if re_build_func_kwargs is None:
            re_build_func(module_name, with_html)
        else:
            re_build_func(**re_build_func_kwargs)

    ext = [i for i in os.listdir() if module_name in i and suffix in i]
    if len(ext) > 0:
        module = import_module(module_name, os.getcwd())
        msg = "The {module_name} module methods:{methods}"
        names = dir(module)
        names = [i for i in names if "__" not in i]
        if log_print:
            print(msg.format(module_name=module_name, methods=names))
        return module
    else:
        raise FileNotFoundError(": There is no './{}.***{}' in '{}',\n".format(module_name, suffix, path),
                                "There are just {},\n".format(os.listdir()),
                                "Please try to build=Ture again.")



"""
Quick build for pyx and cpp file.
used for Rapid debugging.

Passed only for linux !!!

After build=True, the temporary files such as setup.py in local, could be re-write further.
and select the required parts text  and add to your module main ``setup.py`` file.

Include:
---------
``DraftPybind11``
    For cpp (optional, with ``Eigen``).
``DraftPyx``
    For pyx for cython.
``DraftTorch``
    For cpp (optional, with ``torch``)
``TorchJit``
    For cpp (with ``torch``)
``TorchJitInLine``
    For cpp text (with ``torch``)

Examples
---------
>>> from mgetool.draft import DraftPyx

>>> bd = DraftPyx("hello.pyx")
>>> bd.write()
>>> a= bd.quick_import(build=True, with_html=True)
>>> # bd.remove()

Examples
---------

>>> from mgetool.draft import DraftPybind11

>>> bd = DraftPybind11("speedup_pybind11.cpp",pybind11_path=r"/home/iap13/wcx/pybind11",
>>>                    eigen_path=r"/home/iap13/wcx/eigen-3.3.9")
>>> bd.write(functions=["cubic","fcc","bcc"])
>>> a= bd.quick_import(build=True)
>>> # bd.remove()


Examples
---------

>>> from mgetool.draft import TorchJit
>>> bd = TorchJit("speedup_torch.cpp",warm_start=True)
>>> a= bd.quick_import(build=False)

"""

import os
import shutil
import warnings
from abc import abstractmethod
from pathlib import Path

from mgetool.quick_import import quick_import
from mgetool.tool import def_pwd, get_name_without_suffix


class BaseDraft:
    """Base draft for quick test."""

    def __init__(self, file, path=None, temps="temps", warm_start=False, only_file=True, log_print=True):
        """
        Add one temps dir and copy all the file to this disk to escape pollution.

        Parameters
        ----------
        file:str
            file name without path.
        path:str
            path of file.
        temps: str
            Add one temps dir and copy all the file to this disk to escape pollution.
        warm_start:bool
            start from exist file.
        only_file:bool
            just copy the source file to temps.
        log_print:bool
            print the log or not.
        """
        self.log_print = log_print
        self.init_path = os.getcwd()
        # check file
        if path:
            assert r"/" not in file, "Path must in one of `path` parameter or the `filename`."
            assert r"\\" not in file, "Path must in one of `path` parameter or the `filename`."
        else:
            if "/" in file or "\\" in file:
                path = Path(file).parent
                file = os.path.split(file)[-1]

        self.file = file
        self.check_suffix(self.file)

        if path is not None:
            def_pwd(path)
        if os.path.isfile(file):
            pass
        else:
            raise IOError("No file named {} in {}, please re-site your path".format(file, os.getcwd()))

        MODULE_DIR = Path().absolute()
        # temps:
        if temps:
            if warm_start:
                if not os.path.isdir(temps):
                    raise FileNotFoundError("Try to warm start but without no exist {} found".format(temps))
            else:

                files = os.listdir()
                if temps in files:
                    files.remove(temps)
                if only_file:
                    files = [self.file, ]

                if not os.path.isdir(temps):
                    os.mkdir(temps)
                else:
                    warnings.warn("There is exist {temps}. Duplicate files will be overwritten."
                                  "please use remove() to delete temporary file after test.".format(temps=temps))
                for i in files:
                    if os.path.isdir(i):
                        if os.path.isdir(MODULE_DIR / temps / i):
                            shutil.rmtree(MODULE_DIR / temps / i)
                        shutil.copytree(i, MODULE_DIR / temps / i)
                    else:
                        shutil.copy(i, temps)

            self.temps = temps
            self.path = MODULE_DIR / temps

        else:
            self.temps = None
            self.path = MODULE_DIR
        # check module_name
        module_name = get_name_without_suffix(os.path.split(self.file)[-1])
        self.module_name = module_name
        self.build = True
        os.chdir(self.init_path)

    @abstractmethod
    def _suffix(self):
        return "pyx"

    def check_suffix(self, file):
        file = file.split(".")[-1]
        if file == self._suffix():
            pass
        else:
            raise NameError("Just accept {} file".format(self._suffix()))

    def write(self, *args, **kwargs):
        """
        Write setup.py (and optional add tail or head to cpp file.)

        args and kwargs in text_tail:

        Parameters
        ----------
        args:
            parameter for tail.
        kwargs:
            parameter for tail.

            doc="module Document". Document of module.

            functions=["fcc", "bcc", "cubic"]. Method of module, keep same with cpp functions.

        """
        os.chdir(self.path)
        # setup.py
        try:
            setup = self.text_setup(self.module_name, self.file)
            if setup is not None:
                print("Write 'setup.py' for {}.".format(self.module_name))
                setup = self.text_setup(self.module_name, self.file)
                with open(r"./setup.py", "w") as f:
                    f.write(setup)
                print("Make new 'setup.py' successfully.")
            else:
                raise NotImplementedError
        except NotImplementedError:
            pass

        try:
            head = self._text_head()
            if head is not None:
                print("Add head text to {}".format(self.file))

                if self._suffix() == "cpp" or self._suffix() == "c":

                    with open(self.file, "r+") as f:
                        lines = f.readlines()
                        i = 0
                        z = 0
                        while "#" == lines[i][0] or z == 0:
                            if "#" != lines[i][0]:
                                z = 0
                            else:
                                z = 1
                            i += 1
                        head.reverse()
                        [lines.insert(i, h) for h in head if h not in lines]
                        f.seek(0)
                        f.truncate()
                        f.writelines(lines)

                elif self._suffix() == "pyx":
                    with open(self.file, "r+", encoding="UTF-8") as f:
                        lines = f.readlines()
                        if "language_level" in lines[0]:
                            pass
                        else:
                            [lines.insert(0, h) for h in head if h not in lines]
                            f.seek(0)
                            f.truncate()
                            f.writelines(lines)
                else:
                    raise NotImplementedError

        except NotImplementedError:
            pass

        try:
            tail = self._text_tail(self.module_name, *args, **kwargs)
            if tail is not None:
                print("Add tail text to {}".format(self.file))
                with open(self.file, "a+") as f:
                    f.seek(0)
                    strs = f.readlines()
                    strs30 = strs[-30:] if len(strs) > 30 else strs
                    if any([True if 'PYBIND11_MODULE' in i else False for i in strs30]):
                        strs_new = []
                        for i in strs:
                            if 'PYBIND11_MODULE' not in i:
                                strs_new.append(i)
                            else:
                                break
                        f.seek(0)
                        f.truncate()
                        f.writelines(strs_new)

                    f.write(tail)
        except NotImplementedError:
            pass

        os.chdir(self.init_path)

        # end

    @abstractmethod
    def _text_head(self, *args, **kwargs):
        """
        Write head text in cpp file automatically for temporary test.
        for pybind11.
        """

    @abstractmethod
    def _text_tail(self, *args, **kwargs):
        """
        Write tail text in cpp file automatically for temporary test.
        for pybind11.
        """

    @abstractmethod
    def _text_setup(self, *args, **kwargs):
        """"""

    def text_setup(self, *args, **kwargs):
        """
        Write setup.py file for f_pyx automatically for temporary test.
        And the setup.py could be costumed further by yourself.
        All the data is in the same folder.
        """
        return self._text_setup(*args, **kwargs)

    def quick_import(self, build=False, suffix=None, with_html=False):
        os.chdir(self.path)
        self.build = build
        if self._suffix() != "pyx":
            with_html = False
        mod = quick_import(self.module_name, path=self.path, build=build, suffix=suffix, with_html=with_html,
                           log_print=self.log_print)
        os.chdir(self.init_path)
        if self.log_print:
            print("Move back to {}".format(self.init_path))
        return mod

    def remove(self, numbers=None):
        os.chdir(self.path)
        """remove files, beyond retrieve, carefully use!!!"""
        if self.temps is None and numbers is None:
            warnings.warn("Difficult to determine which file to delete. please pass the numbers, such as [0,1,2],"
                          "Please confirm the serial number")
            print("Exist file: {}")
            for i in zip(range(len(os.listdir())), os.listdir()):
                print(i)
        elif self.temps is None and numbers is not None:
            files = [os.listdir()[i] for i in numbers]
            for i in files:
                if os.path.isdir(i):
                    print("dir {} is deleted.".format(i))
                    shutil.rmtree(i)
                else:
                    print("file {} is deleted.".format(i))
                    os.remove(i)
            if self.build is False:
                print(
                    "! Please commented out or delete this line in file to prevent mistaken delete in next time running\n"
                    "Such as:\n"
                    " >>> # bd.remove([6,7,8,9,10])")
            else:
                print("Delete end")
        else:
            shutil.rmtree(self.path)
            print("Delete end")

        os.chdir(self.init_path)


class DraftPybind11(BaseDraft):
    """
    Build for cpp by Pybind11.

    Examples
    -----------

    >>> from mgetool.draft import DraftPybind11

    >>> bd = DraftPybind11("speedup_pybind11.cpp", pybind11_path=r"/home/iap13/wcx/pybind11",
    >>>                    eigen_path=r"/home/iap13/wcx/eigen-3.3.9")
    >>> bd.write(functions=["cubic","fcc","bcc"])
    >>> a= bd.quick_import(build=True)
    >>> # bd.remove()
    """

    def __init__(self, *args, pybind11_path=r"/home/iap13/wcx/pybind11",
                 eigen_path=r"/home/iap13/wcx/eigen-3.3.9", **kwargs):
        """

        Parameters
        ----------
        args
        pybind11_path:
            path of pybind11
        eigen_path:
            path of eigen_path, if you are not using it, please pass '.'
        kwargs
        """
        super(DraftPybind11, self).__init__(*args, **kwargs)

        self.pybind11_path = pybind11_path

        self.eigen_path = "." if eigen_path is None else eigen_path
        if self.log_print:
            print("Check and re-set you path:")
            print("pybind11_path:{}".format(pybind11_path))
            print("eigen_path:{}".format(pybind11_path))

    def _suffix(self):
        return "cpp"

    def _text_setup(self, module_name, file):
        pybind11_path = self.pybind11_path
        eigen_path = self.eigen_path
        setup_text = """
import warnings
warnings.filterwarnings(
    "ignore", "Distutils was imported before Setuptools. This usage is discouraged and may exhibit undesirable behaviors or errors. Please use Setuptools' objects directly or at least import Setuptools first.",  UserWarning, "setuptools.distutils_patch")

from pybind11.setup_helpers import Pybind11Extension, build_ext
from setuptools import setup, find_packages

class BuildExt(build_ext):
    def build_extensions(self):
        self.compiler.compiler_so.remove('-Wstrict-prototypes')
        super(BuildExt, self).build_extensions()
        
        
if "cpp" in "{f_cpp}":
    b_ext = BuildExt
else:
    b_ext = build_ext


ext_modules = []

# pybind11
include_dirs = [r'{pybind11_path}',] 
if "{eigen_path}" == "None" or "{eigen_path}" == ".":
    pass
else:
    include_dirs.append(r'{eigen_path}')  # if need

ext_modules.append(Pybind11Extension("{module_name}", ["{f_cpp}"], language="c++", include_dirs=include_dirs))


def main():
    setup(name="{module_name}",
          version="0.0.1",
          cmdclass={{"build_ext": b_ext}},
          description="Python / C library ",
          author="boliqq07",
          author_email="98988989@qq.com",
          packages=find_packages(exclude=[".tests", ".tests.", "tests.", "tests"]),
          platforms=[
              "Windows",
              "Unix",
          ],
          classifiers=[
              "Development Status :: 4 - Beta",
              "Intended Audience :: Science/Research",
              "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
              "Natural Language :: English",
              "Operating System :: Microsoft :: Windows",
              "Operating System :: Unix",
              "Programming Language :: Python :: 3.6",
              "Programming Language :: Python :: 3.7",
              "Programming Language :: Python :: 3.8",
              "Programming Language :: Python :: 3.9",
          ],
          include_dirs=None,
          # include_dirs=include_dirs,
          ext_modules=ext_modules)

if __name__ == "__main__":
    main()
# `Next in cmd`
# local:
# python setup.py build_ext --inplace
# total:
# python setup.py bdist_wheel

        """.format(module_name=module_name, f_cpp=file, pybind11_path=pybind11_path, eigen_path=eigen_path)

        return setup_text

    def _text_head(self):
        return ["#include <pybind11/pybind11.h>\n",
                "#include <pybind11/stl.h>\n",
                "namespace py = pybind11;\n"
                ]

    def _text_tail(self, module_name, doc="module Document",
                   functions=("fcc", "bcc", "cubic")) -> str:
        """
        Add following code to f_cpp file, Just for function, rather than class.

        Add following code to f_cpp file.''

        PYBIND11_MODULE(module_name, m) {
        m.doc() = "module Document"; // optional module docstring
        m.def("cubic", &cubic, "Document");
        m.def("fcc", &fcc, "Document");
        m.def("bcc", &bcc, "Document");
        }
        ''


        Parameters
        ----------
        module_name:str
            module name, please make sure the name just contain [a-z][0-9] and _. such as "a", default using the name of f_cpp.
        doc:str
            module document.
        functions:list of str
            function name, the names must be function in cpp file.
        """

        func_str = ['m.def("{i}", &{i}, "function Document"); // functions'.format(i=i) for i in functions]
        func_str = "\n".join(func_str)

        PYBIND11_MODULE = r"""
PYBIND11_MODULE({module_name}, m) {{
m.doc() = "{doc}"; // optional module docstring
{func_str}
}}""".format(module_name=module_name, doc=doc, func_str=func_str)
        return PYBIND11_MODULE


class DraftPyx(BaseDraft):
    """
    Build for Pyx.

    Examples
    ---------
    >>> from mgetool.draft import DraftPyx

    >>> bd = DraftPyx("hello.pyx")
    >>> bd.write()
    >>> a= bd.quick_import(build=True, with_html=True)
    >>> # bd.remove()

    Examples
    ---------
    >>> from mgetool.draft import DraftPyx

    >>> bd = DraftPyx("hello.pyx",language="c++")
    >>> bd.write()
    >>> a= bd.quick_import(build=True, with_html=True)
    >>> # bd.remove()
    """

    def __init__(self, *args, language="c", **kwargs):
        super(DraftPyx, self).__init__(*args, **kwargs)
        self.language = language

    def _text_setup(self, module_name, file):
        setup_text = """
import warnings
warnings.filterwarnings(
    "ignore", "Distutils was imported before Setuptools. This usage is discouraged and may exhibit undesirable behaviors or errors. Please use Setuptools' objects directly or at least import Setuptools first.",  UserWarning, "setuptools.distutils_patch")
from setuptools import setup
from Cython.Build import cythonize

import numpy

include_path = [numpy.get_include()]  # just for cimport numpy

def main():

    setup(
        name='{module_name}',
        ext_modules=cythonize('{f_pyx}',include_path=include_path),)

if __name__ == "__main__":
    main()
# `Next in cmd`
# local:
# python setup.py build_ext --inplace
# total:
# python setup.py bdist_wheel

""".format(module_name=module_name, f_pyx=file, language=self.language)

        return setup_text

    def _text_head(self, *args, **kwargs):
        if self.language=="c++":
            return [
            "# distutils: language = c++\n",
            "# cython: language_level=3\n",
            ]
        else:
            return []


class DraftTorch(BaseDraft):
    """
    Torch draft. Passed only for linux !!!

    Examples:
    -----------
    >>> from mgetool.draft import DraftTorch
    >>>
    >>> bd = DraftTorch("speedup_torch.cpp")
    >>> bd.write(functions=["d_sigmoid",])
    >>> a= bd.quick_import(build=True)
    >>> bd.remove()

    """

    def __init__(self, *args, **kwargs):
        super(DraftTorch, self).__init__(*args, **kwargs)

    def _suffix(self):
        return "cpp"

    def _text_tail(self, module_name, doc="module Document",
                   functions=("fcc", "bcc", "cubic")) -> str:
        """
        Add following code to f_cpp file, Just for function, rather than class.

        Add following code to f_cpp file.''

        PYBIND11_MODULE(module_name, m) {
        m.doc() = "module Document"; // optional module docstring
        m.def("cubic", &cubic, "Document");
        m.def("fcc", &fcc, "Document");
        m.def("bcc", &bcc, "Document");
        }
        ''


        Parameters
        ----------
        module_name:str
            module name, please make sure the name just contain [a-z][0-9] and _. such as "a", default using the name of f_cpp.
        doc:str
            module document.
        functions:list of str
            function name, the names must be function in cpp file.
        """

        func_str = ['m.def("{i}", &{i}, "function Document"); // functions'.format(i=i) for i in functions]
        func_str = "\n".join(func_str)

        PYBIND11_MODULE = r"""
PYBIND11_MODULE({module_name}, m) {{
m.doc() = "{doc}"; // optional module docstring
{func_str}
}}""".format(module_name=module_name, doc=doc, func_str=func_str)
        return PYBIND11_MODULE

    def _text_head(self):
        return [
                ]

    def text_setup(self, module_name, f_cpp):

        setup_text = """
import warnings
warnings.filterwarnings("ignore", "Distutils was imported before Setuptools. This usage is discouraged and may exhibit undesirable behaviors or errors. Please use Setuptools' objects directly or at least import Setuptools first.",  UserWarning, "setuptools.distutils_patch")

from setuptools import setup, find_packages
from torch.utils import cpp_extension
import platform

sys_name = platform.system()



class BuildExt(cpp_extension.BuildExtension):
    def build_extensions(self):
        self.compiler.compiler_so.remove('-Wstrict-prototypes')
        super(BuildExt, self).build_extensions()
        
if sys_name == "Linux":
    if "cpp" in "{f_cpp}":
        b_ext = BuildExt
    else:
        b_ext = cpp_extension.BuildExtension
else:
    b_ext = cpp_extension.BuildExtension

ext_modules = []


include_dirs = cpp_extension.include_paths()
library_paths = cpp_extension.library_paths()
extra_link_args = []
for i in library_paths:
    extra_link_args.append('-Wl,-rpath={{}}'.format(i))

ext_modules.append(cpp_extension.CppExtension("{module_name}", ["{f_cpp}"], language="c++",
include_dirs=include_dirs,extra_link_args=extra_link_args))

def main():
    setup(name="{module_name}",
          version="0.0.1",
          cmdclass={{"build_ext": b_ext}},
          description="Python / C library ",
          author="boliqq07",
          author_email="98988989@qq.com",
          packages=find_packages(exclude=[".tests", ".tests.", "tests.", "tests"]),
          platforms=[
              "Windows",
              "Unix",
          ],
          classifiers=[
              "Development Status :: 4 - Beta",
              "Intended Audience :: Science/Research",
              "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
              "Natural Language :: English",
              "Operating System :: Microsoft :: Windows",
              "Operating System :: Unix",
              "Programming Language :: Python :: 3.6",
              "Programming Language :: Python :: 3.7",
              "Programming Language :: Python :: 3.8",
              "Programming Language :: Python :: 3.9",
          ],
          #include_dirs=None,
           include_dirs=include_dirs,
          ext_modules=ext_modules)

if __name__ == "__main__":
    main()
# `Next in cmd`
# local:
# python setup.py build_ext --inplace
# total:
# python setup.py bdist_wheel

            """.format(module_name=module_name, f_cpp=f_cpp)

        return setup_text


class TorchJit(BaseDraft):
    """This type is one JIT of torch, with no setup.py file.  Passed only for linux !!!

    Examples
    -----------
    >>> from mgetool.draft import TorchJit

    >>> bd = TorchJit("speedup_torch.cpp")
    >>> bd.write(functions=["d_sigmoid",])
    >>> a= bd.quick_import(build=True)
    >>> bd.remove()
    """

    def __init__(self, file, path=None, temps="temps", warm_start=False, **kwargs):
        super(TorchJit, self).__init__(file, path=path, temps=temps, warm_start=warm_start, **kwargs)
        try:
            import ninja, torch
        except ImportError:
            raise ImportError("ninja and torch module are need,try turn to 'pip install ninja'.")

    def _suffix(self):
        return "cpp"

    def _text_tail(self, module_name, doc="module Document",
                   functions=("fcc", "bcc", "cubic")) -> str:
        """
        Add following code to f_cpp file, Just for function, rather than class.

        Add following code to f_cpp file.''

        PYBIND11_MODULE(module_name, m) {
        m.doc() = "module Document"; // optional module docstring
        m.def("cubic", &cubic, "Document");
        m.def("fcc", &fcc, "Document");
        m.def("bcc", &bcc, "Document");
        }
        ''


        Parameters
        ----------
        module_name:str
            module name, please make sure the name just contain [a-z][0-9] and _. such as "a", default using the name of f_cpp.
        doc:str
            module document.
        functions:list of str
            function name, the names must be function in cpp file.
        """

        func_str = ['m.def("{i}", &{i}, "function Document"); // functions'.format(i=i) for i in functions]
        func_str = "\n".join(func_str)

        PYBIND11_MODULE = r"""
PYBIND11_MODULE({module_name}, m) {{
m.doc() = "{doc}"; // optional module docstring
{func_str}
}}""".format(module_name=module_name, doc=doc, func_str=func_str)
        return PYBIND11_MODULE

    def quick_import(self, build=False, suffix=None, with_html=False):
        os.chdir(self.path)
        self.build = build
        from torch.utils import cpp_extension

        def re_build_func_torch():
            mod = cpp_extension.load(
                name=self.module_name,
                sources=[self.file, ],
                verbose=True,
                build_directory=self.path,
                is_python_module=True, )
            return mod

        mod = quick_import(self.module_name, path=None, build=build, suffix=suffix,
                           re_build_func=re_build_func_torch, re_build_func_kwargs={}, log_print=self.log_print)
        os.chdir(self.init_path)
        if self.log_print:
            print("Move back to {}".format(self.init_path))
        return mod


class TorchJitInLine:
    """This type is one JIT of torch, with no setup.py file. passed for linux.

    Examples
    -----------
    source = "

    #include <torch/extension.h>

    #include <iostream>

    torch::Tensor d_sigmoid(torch::Tensor z) {

    auto s = torch::sigmoid(z);

    return (1 - s) * s;}
    "

    >>> from mgetool.draft import TorchJitInLine

    >>> bd = TorchJitInLine(source=source)
    >>> bd.write(functions=["d_sigmoid",])
    >>> a= bd.quick_import(build=True)
    >>> bd.remove()

    """

    def __init__(self, source, module_name="TORCH_EXTENSION_NAME", path=None, temps="temps", warm_start=False,
                 log_print=True
                 ):
        """
        Add one temps dir and copy all the file to this disk to escape pollution.

        Parameters
        ----------
        source:str
            all the cpp source text but not file name.
        module_name:
            name of module
        path:str
            path to build.
        temps: str
            Add one temps dir.
        warm_start:bool
            start from exist file.
        """
        self.log_print = log_print
        self.init_path = os.getcwd()
        # check file
        if module_name == "TORCH_EXTENSION_NAME":
            warnings.warn("please re set your module name",NameError)
        self.source = source

        if path is not None:
            def_pwd(path)

        MODULE_DIR = Path().absolute()
        # temps:
        if temps:
            if warm_start:
                if not os.path.isdir(temps):
                    raise FileNotFoundError("Try to warm start but without no exist {} found".format(temps))
            else:

                if not os.path.isdir(temps):
                    os.mkdir(temps)
                else:
                    warnings.warn("There is exist {temps}. Duplicate files will be overwritten."
                                  "please use remove() to delete temporary file after each test.".format(temps=temps))

            self.temps = temps
            self.path = MODULE_DIR / temps

        else:
            self.temps = None
            self.path = MODULE_DIR
        # check module_name
        self.module_name = module_name
        self.build = True
        self.functions = []
        os.chdir(self.init_path)

    def write(self, functions: list):
        self.functions = functions
        if len(functions) == 0:
            print("function names must be set")

    def quick_import(self, build=False, suffix=None):
        os.chdir(self.path)
        self.build = build
        from torch.utils import cpp_extension

        def re_build_func_torch():
            mod = cpp_extension.load_inline(
                name=self.module_name,
                cpp_sources=self.source,
                functions=self.functions,
                verbose=True,
                build_directory=self.path,
                is_python_module=True, )
            return mod

        mod = quick_import(self.module_name, path=self.path, build=build, suffix=suffix,
                           re_build_func=re_build_func_torch, re_build_func_kwargs={}, log_print=self.log_print)
        os.chdir(self.init_path)
        if self.log_print:
            print("Move back to {}".format(self.init_path))
        return mod

    def remove(self, numbers=None):
        """remove files, beyond retrieve, carefully use!!!"""
        os.chdir(self.path)
        if self.temps is None and numbers is None:
            warnings.warn("Difficult to determine which file to delete. please pass the numbers, such as [0,1,2],"
                          "Please confirm the serial number")
            print("Exist file: {}")
            for i in zip(range(len(os.listdir())), os.listdir()):
                print(i)
        elif self.temps is None and numbers is not None:
            files = [os.listdir()[i] for i in numbers]
            for i in files:
                if os.path.isdir(i):
                    print("dir {} is deleted.".format(i))
                    shutil.rmtree(i)
                else:
                    print("file {} is deleted.".format(i))
                    os.remove(i)
            if self.build is False:
                print(
                    "! Please commented out or delete this line in file to prevent mistaken delete "
                    "in next time running\n"
                    "Such as:\n"
                    " >>> # bd.remove([6,7,8,9,10])")
            else:
                print("Delete end")
        else:
            shutil.rmtree(self.path)
            print("Delete end")
        os.chdir(self.init_path)



from mgetool.tool import parallelize, tt, parallelize_imap, batch_parallelize
import numpy as np


def func(n, _=None):
    # time.sleep(0.0001)
    s = np.random.random((10, 50))
    return s

if __name__ == "__main__":

    iterable = np.arange(50)
    s0 = batch_parallelize(4, func, iterable, respective=False, tq=True, mode="m") #无tq
    s1 = parallelize(4, func, iterable, respective=False, tq=True,  mode="j")
    s2 = parallelize_imap(4, func, iterable, tq=True)
    s0 = parallelize(4, func, iterable, respective=False, tq=False, mode="m") #无tq
    s1 = parallelize(4, func, iterable, respective=False, tq=False, mode="j")
    s2 = parallelize_imap(4, func, iterable, tq=False)


    def func(n, _=None):
        # time.sleep(0.0001)
        s = np.random.random((100, 50))
        return s

    iterable = np.arange(10000)

    print("samll loop and big data")

    tt.t
    s0 = batch_parallelize(4, func, iterable, respective=False, tq=True,batch_size=500, mode="m")
    tt.t
    s1 = parallelize(4, func, iterable, respective=False, tq=True,mode="m")
    tt.t
    tt.p



from pathlib import Path

from mgetool.imports import BatchFile
from pymatgen.core import Structure
import numpy as np
import os
name = "Ti"


bf = BatchFile(r"/home/iap13/wcx/featurebox/Instances/Instance_wxx3/data")
os.chdir(Path(__file__).parent)
#
bf.filter_dir_name(include="ed_CONTCAR", layer=-2)
bf.filter_dir_name(include=["Al","Ca","Li"],exclude=["Li"], layer=-2)



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-

# @Time   : 2019/7/29 19:47
# @Author : DEAP
# @Software: PyCharm
# @License: BSD 3-Clause

"""
Notes:
    This file is part of DEAP.
    this is used for pack some function into one box
"""

from inspect import getfullargspec

try:
    from collections.abc import Sequence
except ImportError:
    from collections import Sequence

try:
    from collections.abc import Sequence
except ImportError:
    from collections import Sequence

from copy import deepcopy
from functools import partial, wraps


class Toolbox(object):
    """A toolbox for evolution that contains the evolutionary operators. At
    first the toolbox contains a :meth:`~deap.toolbox.clone` method that
    duplicates any element it is passed as argument, this method defaults to
    the :func:`copy.deepcopy` function. and a :meth:`~deap.toolbox.map`
    method that applies the function given as first argument to every items
    of the iterables given as next arguments, this method defaults to the
    :func:`map` function. You may populate the toolbox with any other
    function by using the :meth:`~deap.base.Toolbox.register` method.

    Concrete usages of the toolbox are shown for initialization in the
    :ref:`creating-types` tutorial and for tools container in the
    :ref:`next-step` tutorial.
    """

    def __init__(self):
        self.register("clone", deepcopy)
        self.register("map", map)

    def register(self, alias, function, *args, **kargs):
        """Register a *function* in the toolbox under the name *alias*. You
        may provide default arguments that will be passed automatically when
        calling the registered function. Fixed arguments can then be overriden
        at function call time.

        :param alias: The name the operator will take in the toolbox. If the
                      alias already exist it will overwrite the the operator
                      already present.
        :param function: The function to which refer the alias.
        :param args: One or more argument (and keyword argument) to pass
                         automatically to the registered function when called,
                         optional.

        The following code block is an example of how the toolbox is used. ::

            def func(a, b, c=3):
                    pass
            tools = Toolbox()
            tools.register("myFunc", func, 2, c=4)
            tools.myFunc(3)


        The registered function will be given the attributes :attr:`__name__`
        set to the alias and :attr:`__doc__` set to the original function's
        documentation. The :attr:`__dict__` attribute will also be updated
        with the original function's instance dictionary, if any.
        """
        pfunc = partial(function, *args, **kargs)
        pfunc.__name__ = alias
        pfunc.__doc__ = function.__doc__

        if hasattr(function, "__dict__") and not isinstance(function, type):
            # Some functions don't have a dictionary, in these cases
            # simply don't copy it. Moreover, if the function is actually
            # a class, we do not want to copy the dictionary.
            pfunc.__dict__.update(function.__dict__.copy())

        setattr(self, alias, pfunc)

    def unregister(self, alias):
        """Unregister *alias* from the toolbox.

        :param alias: The name of the operator to remove from the toolbox.
        """
        delattr(self, alias)

    def decorate(self, alias, *decorators):
        """Decorate *alias* with the specified *decorators*, *alias*
        has to be a registered function in the current toolbox.

        :param alias: The name of the operator to decorate.
        :param decorators: One or more function decorator. If multiple
                          decorators are provided they will be applied in
                          order, with the last decorator decorating all the
                          others.

        .. note::
            Decorate a function using the toolbox makes it unpicklable, and
            will produce an error on pickling. Although this limitation is not
            relevant in most cases, it may have an impact on distributed
            environments like multiprocessing.
            A function can still be decorated manually before it is added to
            the toolbox (using the @ notation) in order to be picklable.
        """
        pfunc = getattr(self, alias)
        function, args, kargs = pfunc.func, pfunc.args, pfunc.keywords
        for decorator in decorators:
            function = decorator(function)
        self.register(alias, function, *args, **kargs)

    def refresh(self, alias=None, *nargs, **nkwargs):
        "the refreshed function  only can be used by kwargs parameter ranther args "

        if isinstance(alias, str):
            if hasattr(self, alias):
                pfunc = getattr(self, alias)
                function, args, kargs = pfunc.func, pfunc.args, pfunc.keywords

                detail = getfullargspec(function)

                n_arg = len(detail.args)
                if detail.defaults:
                    defu = [None] * (n_arg - len(detail.defaults))
                    defu.extend(detail.defaults)  # ?
                else:
                    defu = [None] * n_arg

                defu_dict = {}
                for i, j in zip(detail.args, defu):
                    if j is not None:
                        defu_dict[i] = j

                if detail.kwonlydefaults:
                    defu_dict.update(detail.kwonlydefaults)

                for i, j in zip(detail.args, args):
                    defu_dict[i] = j

                defu_dict.update(kargs)

                defu_dict.update(nkwargs)

                for i, j in zip(detail.args, nargs):
                    if i in defu_dict:
                        del defu_dict[i]

                self.register(alias, function, *nargs, **defu_dict)
            else:
                pass

        elif isinstance(alias, (list, tuple)):
            for i in alias:
                self.refresh(alias=i, *nargs, **nkwargs)


if __name__ == "__main__":
    def func(a, b=1, c=2, name=4, **kwargs):
        print(a, b, c, name, kwargs)
        pass


    def staticLimit(key, max_value):
        def decorator(func):
            @wraps(func)
            def wrapper(*args, **kwargs):
                new_inds = func(*args, **kwargs)

                print(key, max_value)
                return new_inds

            return wrapper

        return decorator


    to = Toolbox()

    to.register("a", func, 1, 5, abss=3)
    to.decorate("a", staticLimit(key=4, max_value=5))
    to.refresh("a", 3, 4, 6, name=3, abss=4)
    to.a()



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-

# @Time   : 2019/7/29 19:48
# @Author : Administrator
# @Software: PyCharm
# @License: BSD 3-Clause


"""
Notes:
    Export data simply.

"""

import os
import sys
from os import remove

import joblib
import numpy as np
import pandas as pd
from skimage import io
from tqdm import tqdm

from mgetool.tool import def_pwd


class Store(object):
    """
    Store file to path.

    Default mode is "w" ,which can be "a+" in txt.

    'w' :
        create a new file, open for writing, clear contents if it exists.

    'a+':
        open for writing, appending to the end of the file if it exists.

    'n':
        create a new file and open it for writing, the name are set by number.

    Example1:
    ----------
    >>> from mgetool.exports import Store
    >>> st =  Store()
    >>> a = ["data1","data2"]
    >>> b = ["data1","data2"]
    >>> st.to_csv(a)
    >>> st.to_txt(b)
    >>> st.to_pkl_pd(b)

    Example2:
    ----------
    Catch the printed to txt.

    >>> from mgetool.exports import Store
    >>> st =  Store()
    >>> st.start()
    >>> print(os.path.dirname(__file__))
    >>> print('------------------')
    >>> for i in tqdm(range(5, 10)):
    >>>     print("this is the %d times" % i)

    >>> st.end()


    Example2:
    ----------
    Catch the printed to txt.

    >>> from mgetool.exports import Store
    >>> st =  Store()
    >>> a = ["data1","data2"]
    >>> b = ["data1","data2"]
    >>> st.to_csv(a)
    >>> st.to_txt(b)
    >>> st.to_pkl_pd(b)
    >>> st.remove(-1) #delete the last file in disk.
    >>> st.withdraw() #delete all the file in disk.
    ...

    See Also:
        ``Store.to_csv``

        ``Store.to_txt``

        ``Store.to_pkl_pd``

    """

    def __init__(self, path=None, filename="filename", prefix: str = None):
        """

        Parameters
        ----------
        path:str
            /data_cluster, or F:data_cluster/data1
        filename:str
            filename
        prefix:str
            prefix for all filenname

        """

        if not prefix:
            prefix = ""
        self._prefix = prefix

        def_pwd(path)

        self._path = path
        self._filename = ""
        self.default_filename = filename
        self._file_list = []

    def __repr__(self):
        return "store to ({}) with {} file".format(self._path, len(self.stored_file))

    __str__ = __repr__

    def _check_name(self, suffix="csv", file_new_name="filename", mode="w"):

        self._filename = file_new_name or self.default_filename

        if "." + suffix in self._filename:
            self._filename = self._filename.replace("." + suffix, "")
        if os.path.isfile('{}{}'.format(self._prefix, self._filename, suffix)) and mode == "n":
            shu1 = 1
            while os.path.isfile('{}{}({}).{}'.format(self._prefix, self._filename, shu1, suffix)):
                shu1 += 1
            self._filename = '{}{}({}).{}'.format(self._prefix, self._filename, shu1, suffix)
        else:
            self._filename = '{}{}.{}'.format(self._prefix, self._filename, suffix)

        if self._filename in self._file_list:
            self._file_list.remove(self._filename)
        self._file_list.append(self._filename)

    def to_csv(self, data, file_new_name=None, mode="w", transposition=False):
        """
        Parameters
        ----------
        data: object
            data.
        file_new_name:str
            file name, if None, default is "filename(i)".
        mode: str
            "w" or "a+"
        transposition:
             transposition the table

        """
        self._check_name("csv", file_new_name, mode=mode)
        if mode == "n":
            mode = "w"
        if isinstance(data, (dict, list)):
            data = pd.DataFrame.from_dict(data).T

        elif isinstance(data, np.ndarray):
            data = pd.DataFrame(data)
        if isinstance(data, pd.DataFrame):
            if transposition:
                data = data.T
            data.to_csv(path_or_buf="%s" % self._filename, sep=",", na_rep='', float_format=None,
                        columns=None, header=True, index=True, index_label=None,
                        mode=mode, encoding=None, )

        else:
            raise TypeError("Not support data_cluster type:%s for csv" % type(data))

    def to_txt(self, data, file_new_name=None, mode="w"):
        """
        Parameters
        ----------
        data: object
            data.
        file_new_name:str
            file name, if None, default is "filename(i)".
        mode: str
            "w" or "a+" or "n"

        """
        self._check_name("txt", file_new_name, mode=mode)
        if mode == "n":
            mode = "w"
        document = open(self._filename, mode=mode)
        document.write(str(data))
        document.close()

    def to_pkl_pd(self, data, file_new_name=None, mode="n"):
        """

        Parameters
        ----------
        data: object
            data.
        mode: str
            ‘n’
        file_new_name:str
            file name, if None, default is "filename(i)".

        """
        self._check_name("pkl_pd", file_new_name, mode=mode)
        pd.to_pickle(data, self._filename)

    def to_pkl_sk(self, data, file_new_name=None, mode="n"):
        """
        DeprecationWarning

        Parameters
        ----------
        data: object
            data.
        mode: str
            ‘n’
        file_new_name:str
            file name, if None, default is "filename(i)".

        """
        self._check_name("pkl_sk", file_new_name, mode=mode)
        joblib.dump(data, self._filename)

    def to_png(self, data, file_new_name=None, mode="w"):
        """

        Parameters
        ----------
        data:object
            data.
        file_new_name:str
            file name, if None, default is "filename(i)".
        mode: str
            ‘n’
        """
        self._check_name("png", file_new_name=file_new_name, mode=mode)
        io.imsave(self._filename, data)

    @classmethod
    def to_multi_file(cls, data_s, suffix="pkl_sk", file_new_name=None, mode="w"):
        """
        Store a series files.

        Parameters
        ----------
        data_s:list of object
            list of data. The single data, should be set in [].
        suffix: str
            file type {"txt": cls.to_txt, "pkl_sk": cls.to_pkl_sk, "pkl_pd": cls.to_pkl_pd, "csv": cls.to_csv,
            "png": cls.to_png}.
        file_new_name:str
            file name, if None, default is "filename(i)".
        mode: str
            "w" , "a+","n".

        """
        dict_func = {"txt": cls.to_txt, "pkl_sk": cls.to_pkl_sk, "pkl_pd": cls.to_pkl_pd,
                     "csv": cls.to_csv, "png": cls.to_png}
        if suffix in ["pkl_sk", "pkl_pd", "png"] and mode == "a+":
            raise UserWarning(
                '"pkl_sk","pkl_pd","png" just accept mode="w" or "n",the file would be stored respectively')

        for data in tqdm(data_s):
            dict_func[suffix](data, file_new_name, mode=mode)

    def remove(self, index_or_name=None):
        """
        Remove the indexed file.

        Parameters
        ----------
        index_or_name:int or str
            index or x_name,default=-1

        """

        if isinstance(index_or_name, str):
            name = index_or_name
            index = -1
        elif isinstance(index_or_name, int):
            name = None
            index = index_or_name
        else:
            name = None
            index = -1
        if not name:
            try:
                files = self._file_list[index]
            except IndexError:
                raise IndexError("No flie or wrong index to remove")
            else:
                if not isinstance(files, list):
                    remove(str(files))
                else:
                    for f in files:
                        remove(str(f))
                del self._file_list[index]

        elif name in self._file_list:
            if not isinstance(name, list):
                remove(str(name))
                self._file_list.remove(name)
            else:
                for f in name:
                    remove(str(f))
                    self._file_list.remove(f)
                self._file_list.remove([])

        else:
            raise NameError("No flie named %s" % index_or_name)

    def withdraw(self):
        """
        Delete all stored_file file.
        """
        files = self._file_list
        for f in files:
            remove(str(f))
        self._file_list = []
        self._filename = ""

    @property
    def stored_file(self):
        """show the stored file"""
        [print(_) for _ in self._file_list]
        return self._file_list

    def start(self, file_new_name="print_log", mode="w"):
        """
        Only for single processing.

        Parameters
        ----------
        file_new_name:str
            file name, if None, default is "filename(i)".
        mode: str
            "w" or "a+" or "n"

        """

        self._check_name("txt", file_new_name, mode=mode)

        if mode == "n":
            mode = "w"

        sys.stdout = Logger(self._filename, mode=mode)

    @staticmethod
    def end():
        if hasattr(sys.stdout, "log"):
            try:
                sys.stdout.log.close()
            except AttributeError:
                pass
        else:
            pass


class Logger(object):
    def __init__(self, filename="Default.log", mode="a+"):
        self.terminal = sys.stdout
        self.log = open(filename, mode)

    def write(self, message):
        self.terminal.write(message)
        self.log.write(message)

    def flush(self):
        pass


if __name__ == "__main__":

    st = Store()
    st.start()

    print(os.path.dirname(__file__))
    print('------------------')
    for i in tqdm(range(5, 10)):
        print("this is the %d times" % i)

    st.end()



# -*- coding: utf-8 -*-

# @Time     : 2021/10/8 15:23
# @Software : PyCharm
# @License  : GNU General Public License v3.0
# @Author   : xxx
import os
import pathlib


def trans_str(string):
    lines = string.split("\\n")
    print(lines)
    lines = "\n".join(lines)
    return lines


def make_batch(paths, cmd="cd $i \necho i \ncd ..", out_file_name="batch.sh"):
    batch_str = """#!/bin/bash
echo $dirname

old_path = $dirname

do

for i in {}

{}

cd old_path

done
        """.format(paths, trans_str(cmd))

    batch_str = batch_str.replace("'", "")
    batch_str = batch_str.replace("[", "")
    batch_str = batch_str.replace("]", "")
    batch_str = batch_str.replace(",", "")
    dirs = os.path.join(os.path.expanduser("~"),"tmp")
    if not os.path.isdir(dirs):
        os.mkdir(dirs)
    os.chdir(dirs)

    bach = open(out_file_name, "w")
    bach.write(batch_str)
    bach.close()
    print("The batch file '{}' is stored in '{}'".format(out_file_name, os.getcwd()))


def make_batch_from_file(path_file, cmd="cd $i \necho i \ncd ..", out_file_name="batch.sh"):
    batch_str = """#!/bin/bash
echo $dirname

old_path = $dirname

for i in $(cat {})

do

{}

cd old_path

done
    """.format(path_file, trans_str(cmd))

    batch_str = batch_str.replace("'", "")
    batch_str = batch_str.replace("[", "")
    batch_str = batch_str.replace("]", "")
    batch_str = batch_str.replace(",", "")

    path = pathlib.Path(path_file).parent
    os.chdir(path)
    bach = open(out_file_name, "w")
    bach.write(batch_str)
    bach.close()
    print("The batch file '{}' is stored in '{}'".format(out_file_name, os.getcwd()))


class CLICommand:
    """
    根据路径文件，创建循环命令， 查看帮助使用 -h。
    可以配合 mgetool findpath 使用，先 findpath 查找路径，再 makebatch 创建循环脚本。
    复杂sh脚本也可以使用该脚本创建，再手动修改具体操作命令。

    如果在 mgetool 中运行:

    Example:

        $ mgetool makebatch -f paths.temp -cmd 'cd $i \necho i \ncd ..'

    如果复制该脚本到某处，仅运行单个脚本:

    Example:

        $ python makebatch.py -f paths.temp -cmd 'cd $i \necho i \ncd ..'

    若希望直接输入路径，请使用 -p 而不是 -f

        $ mgetool makebatch -p “/home/path1 /home/path2” -cmd 'cd $i \necho i \ncd ..'
        $ python makebatch.py -p “/home/path1 /home/path2” -cmd 'cd $i \necho i \ncd ..'


    cmd 命令用单引号。换行使用\n,并且其后不要留有空格。
    """

    @staticmethod
    def add_arguments(parser):
        parser.add_argument('-f', '--path_file', help='source path file', type=str, default="paths.temp")
        parser.add_argument('-p', '--paths', help='source paths', type=str, default=None)
        parser.add_argument('-cmd', '--command', help='command', type=str, default="cd $i \necho i \ncd ..")
        parser.add_argument('-o', '--store_name', help='out file name', type=str, default="batch.sh")

    @staticmethod
    def run(args, parser):
        make_batch_from_file(args.path_file, args.command, args.store_name)


if __name__ == '__main__':
    # 命令行模式
    import argparse

    parser = argparse.ArgumentParser(description="根据路径文件，创建循环命令："
                                                 "python makebatch.py -cmd 'cd $i \necho i \ncd ..'  "
                                                 "cmd 命令用单引号。换行使用\n,并且其后不要留有空格。")
    parser.add_argument('-f', '--path_file', help='source path file', type=str, default="paths.temp")
    parser.add_argument('-p', '--paths', help='source paths', type=str, default=None)
    parser.add_argument('-cmd', '--command', help='command', type=str, default="cd $i \necho i \ncd ..")
    parser.add_argument('-o', '--store_name', help='out file name', type=str, default="batch.sh")
    args = parser.parse_args()
    if args.paths is None:
        make_batch_from_file(args.path_file, args.command, args.store_name)
    else:
        make_batch(args.paths, args.command, args.store_name)



# -*- coding: utf-8 -*-

# @Time     : 2021/8/15 14:09
# @Software : PyCharm
# @License  : GNU General Public License v3.0
# @Author   : xxx

u"""通用画图部分模块
1. 用代码画的图，可以先尝试调用这里的模版画图。
2. 无模版，需要自定义的代码生成的图片，画图代码和图片直接放在自己章节文件夹位置。代码画图推荐使用 plotly, seaborn, networkx, matplotlib.
3. 非代码画图，如使用origin，Matlab, ppt, visio, ps等手动画图，需要上传图片到对应章节！
"""

import re
import networkx as nx
from itertools import product, zip_longest

import matplotlib.gridspec as gridspec
import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
import seaborn as sns
from matplotlib import rcParams
from matplotlib.colorbar import ColorbarBase
from sklearn.datasets import load_boston
from sklearn.metrics import roc_curve, auc
from sklearn.utils import check_random_state
from sympy.physics.quantum.circuitplot import pyplot

markers = ['o', 's', '^', 'v', '<', '>', 'p', 'D', '+', '*', 'H', '1', '2', '3', '4'] * 5
ls = ['-', '-.', '- ', ':'] * 10


def setting_rcParams_single(font="Arial", figure_size=(10, 7.5)):
    """短图通用配置

    本函数无返回值，rcParams为全局变量，调用本函数即可。

    Example:

    >>> setting_rcParams_single() #调用本函数即可
    """

    if font is None or font == "DejaVu Sans":
        pass
    elif font == "Times":
        rcParams['font.family'] = 'serif'
        rcParams['font.serif'] = ['Times new roman']
    elif font == "Arial":
        rcParams['font.family'] = 'sans serif'
        rcParams['font.sans-serif'] = ["Arial"]
    elif font == "Helvetica":
        rcParams['font.family'] = 'sans serif'
        rcParams['font.sans-serif'] = ["Helvetica"]
    elif font == "Tahoma":
        rcParams['font.family'] = 'sans serif'
        rcParams['font.sans-serif'] = ["Tahoma"]
    elif font == "Monospace":
        rcParams['font.family'] = 'Monospace'
    elif font in ["SimHei", "黑体"]:
        rcParams['font.family'] = 'sans serif'
        rcParams['font.sans-serif'] = ["SimHei"]
    elif font in ["SimSun", "宋体"]:
        rcParams['font.family'] = 'serif'
        rcParams['font.serif'] = ["SimSun"]
    elif font == "宋体+Times":
        config = {
            "font.family": 'serif',
            "mathtext.fontset": 'stix',
            "font.serif": ['SimSun'],
        }
        print(r"使用 $\mathrm{This \; is \; one \; sample}$ 包裹英文，变为新罗马，否则仍然为宋体，\; 代表空格。（中文勿改）")
        rcParams.update(config)
    elif font == "宋体+DejaVu":
        config = {
            "font.family": 'serif',
            "font.serif": ['SimSun'],
        }
        print(r"使用 $\mathrm{This \; is \; one \; sample}$ 包裹英文，变为新罗马，否则仍然为宋体，\; 代表空格。（中文勿改）")
        rcParams.update(config)
    else:
        NotImplementedError('just accept ["宋体+Times","宋体","黑体","Times","Arial","Tahoma","Helvetica"]')

    rcParams['axes.unicode_minus'] = False
    rcParams['figure.dpi'] = 300
    rcParams['figure.figsize'] = figure_size

    rcParams['figure.subplot.left'] = 0.1
    rcParams['figure.subplot.right'] = 0.92
    rcParams['figure.subplot.top'] = 0.92
    rcParams['figure.subplot.bottom'] = 0.1

    rcParams['axes.titlesize'] = 24
    rcParams['axes.labelsize'] = 24
    rcParams['axes.linewidth'] = 3

    rcParams['xtick.major.width'] = 3
    rcParams['ytick.major.width'] = 3
    rcParams['xtick.major.size'] = 8
    rcParams['ytick.major.size'] = 8

    rcParams['xtick.labelsize'] = 22
    rcParams['ytick.labelsize'] = 22
    rcParams['xtick.direction'] = 'in'
    rcParams['ytick.direction'] = 'in'

    rcParams['legend.fontsize'] = 20

    rcParams['axes.grid'] = True
    rcParams['grid.alpha'] = 0.15


def setting_rcParams_long(font="Arial", figure_size=(14.5, 11)):
    """长通用配置

    本函数无返回值，rcParams为全局变量，调用本函数即可

    Example:

    >>> setting_rcParams_single() #调用本函数即可
    """

    if font is None or font == "DejaVu Sans":
        pass
    elif font == "Times":
        rcParams['font.family'] = 'serif'
        rcParams['font.serif'] = ['Times new roman']
    elif font == "Arial":
        rcParams['font.family'] = 'sans serif'
        rcParams['font.sans-serif'] = ["Arial"]
    elif font == "Helvetica":
        rcParams['font.family'] = 'sans serif'
        rcParams['font.sans-serif'] = ["Helvetica"]
    elif font == "Tahoma":
        rcParams['font.family'] = 'sans serif'
        rcParams['font.sans-serif'] = ["Tahoma"]
    elif font == "Monospace":
        rcParams['font.family'] = 'Monospace'
    elif font in ["SimHei", "黑体"]:
        rcParams['font.family'] = 'sans serif'
        rcParams['font.sans-serif'] = ["SimHei"]
    elif font in ["SimSun", "宋体"]:
        rcParams['font.family'] = 'serif'
        rcParams['font.serif'] = ["SimSun"]
    elif font == "宋体+Times":
        config = {
            "font.family": 'serif',
            "mathtext.fontset": 'stix',
            "font.serif": ['SimSun'],
        }
        print(r"使用 $\mathrm{This \; is \; one \; sample}$ 包裹英文，变为新罗马，否则仍然为宋体，\; 代表空格。（中文勿改）")
        rcParams.update(config)
    elif font == "宋体+DejaVu":
        config = {
            "font.family": 'serif',
            "font.serif": ['SimSun'],
        }
        print(r"使用 $\mathrm{This \; is \; one \; sample}$ 包裹英文，变为新罗马，否则仍然为宋体，\; 代表空格。（中文勿改）")
        rcParams.update(config)
    else:
        NotImplementedError('just accept ["宋体+Times","宋体","黑体","Times","Arial","Tahoma","Helvetica"]')

    rcParams['axes.unicode_minus'] = False
    rcParams['figure.dpi'] = 300
    rcParams['figure.figsize'] = figure_size

    rcParams['figure.subplot.left'] = 0.09
    rcParams['figure.subplot.right'] = 0.93
    rcParams['figure.subplot.top'] = 0.93
    rcParams['figure.subplot.bottom'] = 0.09

    rcParams['axes.titlesize'] = 16
    rcParams['axes.labelsize'] = 16
    rcParams['axes.linewidth'] = 2

    rcParams['xtick.major.width'] = 2
    rcParams['ytick.major.width'] = 2
    rcParams['xtick.major.size'] = 6
    rcParams['ytick.major.size'] = 6

    rcParams['xtick.labelsize'] = 15
    rcParams['ytick.labelsize'] = 15
    rcParams['xtick.direction'] = 'in'
    rcParams['ytick.direction'] = 'in'

    rcParams['legend.fontsize'] = 16

    rcParams['axes.grid'] = True
    rcParams['grid.alpha'] = 0.15


def setting_rcParams_part(font="Arial", figure_size=(7, 5)):
    """1/2分图中的单个图，通用配置（后续手动再合并为长图）

    本函数无返回值，rcParams为全局变量，调用本函数即可

    Example:

    >>> setting_rcParams_single() #调用本函数即可
    """

    if font is None or font == "DejaVu Sans":
        pass
    elif font == "Times":
        rcParams['font.family'] = 'serif'
        rcParams['font.serif'] = ['Times new roman']
    elif font == "Arial":
        rcParams['font.family'] = 'sans serif'
        rcParams['font.sans-serif'] = ["Arial"]
    elif font == "Helvetica":
        rcParams['font.family'] = 'sans serif'
        rcParams['font.sans-serif'] = ["Helvetica"]
    elif font == "Tahoma":
        rcParams['font.family'] = 'sans serif'
        rcParams['font.sans-serif'] = ["Tahoma"]
    elif font == "Monospace":
        rcParams['font.family'] = 'Monospace'
    elif font in ["SimHei", "黑体"]:
        rcParams['font.family'] = 'sans serif'
        rcParams['font.sans-serif'] = ["SimHei"]
    elif font in ["SimSun", "宋体"]:
        rcParams['font.family'] = 'serif'
        rcParams['font.serif'] = ["SimSun"]
    elif font == "宋体+Times":
        config = {
            "font.family": 'serif',
            "mathtext.fontset": 'stix',
            "font.serif": ['SimSun'],
        }
        print(r"使用 $\mathrm{This \; is \; one \; sample}$ 包裹英文，变为新罗马，否则仍然为宋体，\; 代表空格。（中文勿改）")
        rcParams.update(config)
    elif font == "宋体+DejaVu":
        config = {
            "font.family": 'serif',
            "font.serif": ['SimSun'],
        }
        print(r"使用 $\mathrm{This \; is \; one \; sample}$ 包裹英文，变为新罗马，否则仍然为宋体，\; 代表空格。（中文勿改）")
        rcParams.update(config)
    else:
        NotImplementedError('just accept ["宋体+Times","宋体","黑体","Times","Arial","Tahoma","Helvetica"]')

    rcParams['axes.unicode_minus'] = False
    rcParams['figure.dpi'] = 300
    rcParams['figure.figsize'] = figure_size

    rcParams['figure.subplot.left'] = 0.12
    rcParams['figure.subplot.right'] = 0.93
    rcParams['figure.subplot.top'] = 0.93
    rcParams['figure.subplot.bottom'] = 0.12

    rcParams['axes.titlesize'] = 32
    rcParams['axes.labelsize'] = 32
    rcParams['axes.linewidth'] = 3.5

    rcParams['xtick.major.width'] = 3.5
    rcParams['ytick.major.width'] = 3.5
    rcParams['xtick.major.size'] = 8
    rcParams['ytick.major.size'] = 8

    rcParams['xtick.labelsize'] = 30
    rcParams['ytick.labelsize'] = 30
    rcParams['xtick.direction'] = 'in'
    rcParams['ytick.direction'] = 'in'

    rcParams['legend.fontsize'] = 26

    rcParams['axes.grid'] = True
    rcParams['grid.alpha'] = 0.15


def spilt_chinese_and_english(strings):
    """英文转换为latex"""

    def subs_en(string2):
        uncn = re.compile(r'[^\u4e00-\u9fa5]*')
        string2e = uncn.findall(string2)
        string2e = [i for i in string2e if i != '']
        string2e = [i.replace(" ", " \; ") for i in string2e]
        string2e = ["$\mathrm{{{}}}$".format(i) for i in string2e]

        uncn2 = re.compile(r'[\u4e00-\u9fa5]*')
        string2c = uncn2.findall(string2)
        string2c = [i for i in string2c if i != '']

        string2_ = []
        if string2[0].isascii():
            k = zip_longest(string2c, string2e)
        else:
            k = zip_longest(string2e, string2c)

        for i, j in k:
            string2_.append(j)
            string2_.append(i)
        string2_ = "".join([i for i in string2_ if i is not None])
        return string2_

    if "$" in strings:
        size = len(strings)
        site_result = re.finditer("\$", strings)
        site = [i.start() for i in site_result]
        assert len(site) >= 2, "$必须成对出现！"

        no_site = [0, ]
        no_site.extend(site)
        no_site.append(size)
        site = np.array(site).reshape((-1, 2))
        string1 = [strings[i[0]:i[1] + 1] for i in site]
        no_site = np.array(no_site).reshape((-1, 2))

        no_site[:, 0] += 1
        no_site[0, 0] = 0
        no_site[-1, -1] = size
        string2 = [strings[i[0]:i[1]] for i in no_site]

        string2 = [subs_en(i) for i in string2 if i not in ['', "$"]]
        string_ = []

        if strings[0] == "$":
            k = zip_longest(string2, string1)
        else:
            k = zip_longest(string1, string2)

        for i, j in k:
            string_.append(j)
            string_.append(i)
        string_ = "".join([i for i in string_ if i is not None])

    else:
        string_ = subs_en(strings)

    string_ = string_.replace(r'$\mathrm{ \; }$', " ")

    return string_


class BasePlot(object):
    """
    基础画图。

    >>> bp = BasePlot()
    >>> plt = bp.scatter_45_line(x, y_predict, strx='x', stry='y_predict')
    >>> plt.show()
    >>> plt = bp.line_scatter(y_scatter, y_lines, strx='x', stry='ys')
    >>> plt.show()
    >>> plt = bp.lines(ys)
    >>> plt.show()
    >>> plt = bp.lines(ys,x)
    >>> plt.show()

    """

    def __init__(self, font="Arial", figure_size=(10, 7.5), types="single"):
        """

        Args:
            font: (str), 字体。如：【Arial,DejaVu Sans,宋体+Times,宋体，Times】
            figure_size:(tuple) 图片大小。
            types:(str) 图片种类: 如：【single,long,part】
        """
        if types == "single":
            setting_rcParams_single(font=font, figure_size=figure_size)
        elif types == "long":
            setting_rcParams_long(font=font, figure_size=figure_size)
        elif types == "part":
            setting_rcParams_part(font=font, figure_size=figure_size)
        else:
            setting_rcParams_single(font=font, figure_size=figure_size)

    @staticmethod
    def base_axes():
        fig = plt.figure()
        ax = fig.add_subplot(111)
        return ax, plt

    @staticmethod
    def base_figure():
        plt.figure(0)
        return plt

    @staticmethod
    def scatter(x, y, strx='x', stry='y_predict', line_45=False,color="orange"):
        """
        散点图
        """
        fig = plt.figure()
        ax = fig.add_subplot(111)
        ax.scatter(x, y, marker='o', s=150, alpha=0.7, c=color, linewidths=None, edgecolors='blue')
        if line_45:
            ax.plot([min(x), max(x)], [min(x), max(x)], '--', ms=5, lw=2, alpha=0.7, color='black')
        plt.xlabel(strx)
        plt.ylabel(stry)
        return plt

    def roc_curve(self, y_true, y_predict):
        """ROC 曲线。"""
        fpr, tpr, thresholds = roc_curve(y_true, y_predict)
        roc_auc = auc(fpr, tpr)
        return self.roc(fpr, tpr, roc_auc)

    def roc(self, fpr, tpr, roc_auc):
        """ROC 曲线."""
        # right_index = (tpr + (1 - fpr) - 1)
        # yuzhi = max(right_index)
        # index = right_index.index(max(right_index))
        # tpr_val = tpr(index)
        # fpr_val = fpr(index)

        plt.subplots()
        plt.plot(fpr, tpr, color='darkorange',
                 lw=3, label='ROC curve (area = %0.2f)' % roc_auc)
        plt.plot([0, 1], [0, 1], color='navy', lw=3, linestyle='--')
        plt.xlim([0.0, 1.0])
        plt.ylim([0.0, 1.05])
        plt.xlabel('False Positive Rate')
        plt.ylabel('True Positive Rate')
        plt.title('ROC Curve')
        plt.legend(loc="lower right")
        return plt

    def scatter_45_line(self, y_true, y_predict, strx='x', stry='y_predict', line_45=True):
        """
        散点图45度线
        """
        return self.scatter(x=y_true, y=y_predict, strx=strx, stry=stry, line_45=line_45)

    @staticmethod
    def lines(ys, x=None, ys_labels=None, strx='x', stry='ys', no_marker=False, mark_size_ratio=1.0):
        """
        点线图，线图。

        Args:
            ys: (np.ndarray,with shape (n_sample,n_target)), lines
            x:  (np.ndarray),defualt is rank number.
            ys_labels: lables of y.
            strx: (str) axis name
            stry: (str) axis name
            no_marker: (bool), 删除形状
            mark_size_ratio: (float),形状大小。

        """

        markers_ = markers
        if no_marker:
            markers_ = [None] * 100

        if isinstance(ys, pd.DataFrame):
            ys_labels = list(ys.columns.values)
            ys = ys.values
        fig = plt.figure()
        ax = fig.add_subplot(111)
        ys = np.array(ys)
        if not ys_labels or len(ys_labels) != ys.shape[1]:
            labels = ["y%s" % i for i in range(ys.shape[1])]
        else:
            labels = ys_labels
        if x is None:
            for i in range(ys.shape[1]):
                ax.plot(ys[:, i], lw=2, ms=10 * mark_size_ratio, marker=markers_[i], label=labels[i])
        else:
            x = np.array(x)
            for i in range(ys.shape[1]):
                ax.plot(x, ys[:, i], lw=2, ms=10 * mark_size_ratio, marker=markers_[i], label=labels[i])

        plt.xlabel(strx)
        plt.ylabel(stry)
        ax.legend()
        return plt

    @staticmethod
    def line_scatter(x, y_scatter, y_lines, strx='x', stry='ys'):
        """散点，线图独立，合并画图，一般不用"""
        fig = plt.figure()
        ax = fig.add_subplot(111)
        ax.scatter(x, y_scatter, marker='o', s=70, alpha=0.7, c='blue', linewidths=None, edgecolors='blue')
        ax.plot(y_lines, '-', lw=2, alpha=0.7, color='black')
        plt.xlabel(strx)
        plt.ylabel(stry)
        return plt

    @staticmethod
    def bar(data, types=None, labels=None, strx='x', stry='ys'):
        """
        柱状图, data为 pandas.Fataframe.
        """

        if isinstance(data, pd.DataFrame):
            types = list(data.columns.values)
            labels = list(data.index.values)
            data = data.values

        data = np.array(data)
        if data.ndim == 1:
            data.reshape((-1, 1))

        index = np.arange(data.shape[0]) * data.shape[1] // 2
        bar_width = 0.35

        if not labels:
            labels = ["f%s" % i for i in index]
        if not types:
            types = list(range(data.shape[1]))
        opacity = 0.4

        fig, ax = plt.subplots()
        for i, (x, typei) in enumerate(zip(data.T, types)):
            ax.bar(index + i * bar_width, x, bar_width, label=typei, alpha=opacity)

        plt.xlabel(strx)
        plt.ylabel(stry)
        ax.set_xticks(index + len(types) * bar_width / 2 - bar_width / 2)
        ax.set_xticklabels(labels)
        ax.legend()
        # fig.tight_layout()

        sns.boxplot()
        return plt

    @staticmethod
    def corr_heatmap(data, square=True, linewidths=.5, annot=False):
        """相关系数图, data使用pandas.DataFrame数据，,strx,stry 必须是列名。"""
        fig = plt.figure()
        fig.add_subplot(111)
        # plt.xticks(rotation='90')
        sns.heatmap(data, cmap="seismic", square=square, linewidths=linewidths, annot=annot,
                    xticklabels=True,
                    yticklabels=True)
        return plt

    @staticmethod
    def violin(strx, stry, data):
        """小提琴图, data为 pandas.Fataframe,strx,stry 必须是列名。"""
        fig = plt.figure()
        fig.add_subplot(111)
        sns.violinplot(x=strx, y=stry, data=data,
                       linewidth=2,  # 线宽
                       width=0.8,  # 箱之间的间隔比例
                       palette='hls',  # 设置调色板
                       order=None,  # 筛选类别
                       scale='area',  # 测度小提琴图的宽度：area-面积相同，count-按照样本数量决定宽度，width-宽度一样
                       gridsize=50,  # 设置小提琴图边线的平滑度，越高越平滑
                       # bw = 0.8        # 控制拟合程度，一般可以不设置
                       hue='smoker',  # 分类
                       split=True,  # 设置是否拆分小提琴图
                       inner="quartile"  # 设置内部显示类型 → “box”, “quartile”, “point”, “stick”, None
                       )
        return plt

    @staticmethod
    def box(strx, stry, data):
        """箱线图，data为 pandas.Fataframe，,strx,stry 必须是列名。"""
        sns.boxplot(x=strx, y=stry, data=data,
                    linewidth=2,  # 线宽
                    width=0.8,  # 箱之间的间隔比例
                    fliersize=3,  # 异常点大小
                    palette='hls',  # 设置调色板
                    whis=1.5,  # 设置IQR
                    notch=True,  # 设置是否以中值做凹槽
                    order=['Thur', 'Fri', 'Sat', 'Sun'],  # 筛选类别
                    )

        sns.swarmplot(x=strx, y=stry, data=data, color='k', size=3, alpha=0.8)
        return plt

    @staticmethod
    def yy_jointplot(strx, stry, data):
        """单/双变量画图，data为 pandas.Fataframe，,strx,stry 必须是列名。"""
        sns.jointplot(strx, stry, data,
                      )
        return plt

    @staticmethod
    def imshow(np_array):
        """热点图"""
        fig = plt.figure()
        ax = fig.add_subplot(111)
        ax.imshow(np_array)
        return plt

    def show(self):
        plt.show()

    @staticmethod
    def corr_plot(x_cof, x_name=None, left_down="circle", right_top="pie", threshold_left=0.0, threshold_right=0.0,
                  title=None, label_axis="off", front_raito=1):
        """
        相关系数图,比较耗时，不要超过25个特征。

        Examples
        -----------
        >>> data = load_boston(return_X_y=False)
        >>> name0 = data["feature_names"]
        >>> x = data["data"]
        >>> ys = data["target"]
        >>> x_cof = np.corrcoef(x.T)
        >>> #plot
        >>> plt = corr_plot(x_cof, name0, left_down="circle", right_top="text", threshold_right=0.7, label_axis="off")
        >>> plt.show()

        Parameters
        ----------
        x_cof:np.ndarray
            correlation coefficient matrix
        x_name:list,None
            feature names
        left_down:None,"pie","fill","text","circle"
            type for left_down
        right_top:None,"pie","fill","text","circle"
            type for right_top
        threshold_left:float
            threshold for show.
        threshold_right:float
            threshold for show.
        title:str
            picture title
        label_axis:"left","right","off"
            label_axis
        front_raito:float
            front scare for show
        """
        assert x_cof.shape[0] == x_cof.shape[1]

        x_cof = np.round(x_cof, 2)
        if x_name is None:
            name = ["$x_{{{i}}}$".format(i=i) for i in range(x_cof.shape[1])]

        size = x_cof
        or_size = np.nan_to_num((abs(size) / size) * (1 - abs(size)))

        n = size.shape[0]
        explode = (0, 0)
        gs = gridspec.GridSpec(n, n)
        gs.update(wspace=0, hspace=0)

        cmap = plt.get_cmap("bwr")  # args
        fill_colors = cmap(size / 2 + 0.5)  # args

        fig = plt.figure(figsize=(12, 12), frameon=True)  # args

        title_fontsize = round(22 * front_raito)  # c_args
        ax_fontsize = round(22 * front_raito)
        score_fontsize = round(16 * front_raito)
        circle_size = round(600 * front_raito)

        fig.text(0.5, 0.05, title, fontsize=title_fontsize, horizontalalignment='center',
                 verticalalignment='center')  # zou, xia

        for i, j in product(range(n), range(n)):
            if j < i and abs(size[i, j]) >= threshold_left:
                types = left_down
            elif j > i and abs(size[i, j]) >= threshold_right:
                types = right_top
            else:
                types = None

            if types == "pie":
                ax = plt.subplot(gs[i, j])
                ax.pie((abs(size[i, j]), abs(or_size[i, j])), explode=explode, labels=None, autopct=None, shadow=False,
                       startangle=90,
                       colors=[fill_colors[i, j], 'w'], wedgeprops=dict(width=1, edgecolor='black', linewidth=0.5),
                       counterclock=False,
                       frame=False, center=(0, 0), )
                ax.set_xlim(-1, 1)
                ax.axis('equal')

            elif types == "fill":
                ax = plt.subplot(gs[i, j])
                ax.set_facecolor(fill_colors[i, j])
                [ax.spines[_].set_color('w') for _ in ['right', 'top', 'left', 'bottom']]

                ax.set_xticks([])
                ax.set_yticks([])

            elif types == "fillandtext":
                ax = plt.subplot(gs[i, j])
                ax.set_facecolor(fill_colors[i, j])
                [ax.spines[_].set_color('w') for _ in ['right', 'top', 'left', 'bottom']]

                ax.text(0.5, 0.5, size[i, j],
                        fontdict={"color": "black"},  # args
                        fontsize=score_fontsize,  # c_arg
                        horizontalalignment='center', verticalalignment='center')
            elif types == "text":
                ax = plt.subplot(gs[i, j])
                ax.text(0.5, 0.5, size[i, j],
                        fontdict={"color": "b"},  # args
                        fontsize=score_fontsize,  # c_arg
                        horizontalalignment='center', verticalalignment='center')
                ax.set_xticks([])
                ax.set_yticks([])
                # plt.axis('off')
            elif types == "circle":
                ax = plt.subplot(gs[i, j])
                ax.axis('equal')
                ax.set_xlim(-1, 1)
                ax.scatter(0, 0, color=fill_colors[i, j], s=circle_size * abs(size[i, j]) ** 2)
                ax.set_xticks([])

                ax.set_yticks([])
                # plt.axis('off')

            else:
                pass

        for k in range(n):
            ax = plt.subplot(gs[k, k])

            # ax.axis('equal')
            # ax.set_xlim(-1, 1)
            # ax.scatter(0, 0, color=fill_colors[k, k], s=circle_size * abs(size[k, k]))
            # ax.set_xticks([])
            #
            # ax.set_yticks([])

            ax.text(0.5, 0.5, name[k], fontsize=ax_fontsize, horizontalalignment='center', verticalalignment='center')
            ax.set_xticks([])
            ax.set_yticks([])
            if label_axis == "left":
                color = ["w", "w", "b", "b"]
                [ax.spines[i].set_color(j) for i, j in zip(['right', 'top', 'left', 'bottom'], color)]
            elif label_axis == "right":
                color = ["b", "b", "w", "w"]
                [ax.spines[i].set_color(j) for i, j in zip(['right', 'top', 'left', 'bottom'], color)]
            else:
                plt.axis('off')

        @pyplot.FuncFormatter
        def fake_(x, pos):
            return round(2 * (x - 0.5), 1)

        fig.subplots_adjust(right=0.80)
        cbar_ax = fig.add_axes([0.85, 0.125, 0.03, 0.75])
        ColorbarBase(cbar_ax, cmap=cmap, ticks=[0, 0.25, 0.5, 0.75, 1], format=fake_)
        fig.set_size_inches(9, 8.5, forward=True)
        return plt

    @staticmethod
    def _displacement(binary_distance, print_noise=0.001):
        rd = check_random_state(0)
        q = rd.random_sample(binary_distance.shape) * print_noise / 10
        binary_distance = binary_distance + q

        indexs = np.argwhere(binary_distance <= 0)
        indexs = indexs[np.where(indexs[:, 0] > indexs[:, 1])]
        t = rd.random_sample(indexs.shape[0]) * print_noise / 20
        binary_distance[indexs[:, 0], indexs[:, 1]] = t
        binary_distance[indexs[:, 1], indexs[:, 0]] = t
        return binary_distance


    def distance(self, node_color, edge_color_pen=0.7, binary_distance=None, print_noise=0.001,
                 node_name=None, highlight=None):
        """
        kamada_kawai_lay.

        Parameters
        ----------
        highlight:list
            change shape
        slices: list
            the lists of the index of feature subsets, each feature subset is a node.
            Examples 3 nodes
            [[1,4,5],[1,4,6],[1,2,7]]
        node_color: np.ndarray 1D, list, the same size as slices
            the label to classify the node
        edge_color_pen: int
            the transparency of edge between node
        binary_distance: np.ndarray
            distance matrix for each pair node
        print_noise: int
            add noise for less printing overlap
        node_name: list
            x_name of node
        """
        plt.figure()
        g = nx.Graph()

        def _my_ravel(data_cof):
            for i in range(data_cof.shape[0]):
                for k in range(i + 1, data_cof.shape[0]):
                    yield i, k, data_cof[i, k]

        distances = self._displacement(binary_distance, print_noise=print_noise)

        distance_weight = list(_my_ravel(distances))
        g.add_weighted_edges_from(distance_weight)
        # edges=nx.get_edge_attributes(g, 'weight').items()
        edges, weights = zip(*nx.get_edge_attributes(g, 'weight').items())
        weights = weights / max(weights)
        pos = nx.layout.kamada_kawai_layout(g)  # calculate site

        if node_name is None:
            le = binary_distance.shape[0]
            lab = {i: i for i in range(le)}

        elif node_name is False:
            lab=None

        else:
            assert binary_distance.shape[0]
            if isinstance(node_name, list) and isinstance(node_name[0], list):
                strr = ","
                node_name = [strr.join(i) for i in node_name]
            lab = {i: j for i, j in enumerate(node_name)}

        nodesize = [600] * distances.shape[0]
        node_edge_color = ["w"] * distances.shape[0]
        if highlight:
            for i in highlight:
                node_edge_color[i] = "aqua"
            for i in highlight:
                nodesize[i] *= 1.3

        mp = {-1: 'gray', 0: "mediumpurple", 1: 'seagreen', 2: 'goldenrod', 3: 'deeppink', 4: "chocolate",
              5: "lightseagreen", }
        node_color = list(map(lambda x: mp[x], node_color))

        nx.draw(g, pos, edgelist=edges, edge_color=np.around(weights, decimals=3) ** edge_color_pen,
                edge_cmap=plt.cm.Blues_r, edge_vmax=0.7,
                width=weights,
                labels=lab, font_size=12,
                node_color=np.array(node_color), vmin=-1,
                edgecolors=node_edge_color, linewidths=1,
                node_size=nodesize,
                )

        return plt



# -*- coding: utf-8 -*-

# @Time     : 2021/10/8 14:29
# @Software : PyCharm
# @License  : GNU General Public License v3.0
# @Author   : xxx
import os

from mgetool.imports import BatchFile


def run(args, parser):
    # args = args.parse_args()
    bf = BatchFile(args.path, suffix=args.suffix)
    bf.filter_dir_name(include=args.dir_include, exclude=args.dir_exclude, layer=args.layer)
    bf.filter_file_name(include=args.file_include, exclude=args.file_exclude)
    bf.merge()

    fdir = bf.file_dir
    fdir.sort()
    os.chdir(args.path)
    if not args.abspath:
        absp = os.path.abspath(args.path)
        fdir = [i.replace(absp, ".") for i in fdir]
    with open(str(args.store_name), mode="w") as f:
        fdir = "\n".join(fdir)
        f.writelines(fdir)

    print("The paths '{}' are stored in '{}'.".format(args.store_name,str(os.getcwd())))
    print("OK")


class CLICommand:
    """
    寻找符合要求所有的子路径, 查看参数帮助使用 -h。

    如果在 mgetool 中运行:

    Example:

        $ mgetool findpath -p /home/dir_name -if POSCAR

    如果复制该脚本到某处，仅运行单个脚本:

    Example:

        $ python findpath.py -p /home/dir_name -if POSCAR
    """

    @staticmethod
    def add_arguments(parser):
        parser.add_argument('-p', '--path', help='source path', type=str, default=".")
        parser.add_argument('-s', '--suffix', help='suffix of file', type=str, default=None)
        parser.add_argument('-if', '--file_include', help='include file name.', type=str, default=None)
        parser.add_argument('-ef', '--file_exclude', help='exclude file name.', type=str, default=None)
        parser.add_argument('-id', '--dir_include', help='include dir name.', type=str, default=None)
        parser.add_argument('-ed', '--dir_exclude', help='exclude dir name.', type=str, default=None)
        parser.add_argument('-l', '--layer', help='dir depth, default the last layer', type=int, default=-1)
        parser.add_argument('-abspath', '--abspath', help='return abspath', type=bool, default=False)
        parser.add_argument('-o', '--store_name', help='out file name, default paths.temp', type=str, default="paths.temp")

    @staticmethod
    def run(args, parser):
        run(args,parser)


if __name__ == '__main__':
    # 命令行模式
    import argparse

    parser = argparse.ArgumentParser(description="根据路径文件，寻找所有符合要求的文件子路径："
                                                 "python findpath.py -p /home/dir_name -if POSCAR"
                                                 )

    parser.add_argument('-p', '--path', help='source path', type=str, default=".")
    parser.add_argument('-s', '--suffix', help='suffix of file', type=str, default=None)
    parser.add_argument('-if', '--file_include', help='include file name.', type=str, default=None)
    parser.add_argument('-ef', '--file_exclude', help='exclude file name.', type=str, default=None)
    parser.add_argument('-id', '--dir_include', help='include dir name.', type=str, default=None)
    parser.add_argument('-ed', '--dir_exclude', help='exclude dir name.', type=str, default=None)
    parser.add_argument('-l', '--layer', help='dir depth,default the last layer', type=int, default=-1)
    parser.add_argument('-abspath', '--abspath', help='return abspath', type=bool, default=False)
    parser.add_argument('-o', '--store_name', help='out file name,default paths.temp', type=str, default="paths.temp")
    args = parser.parse_args()
    run(args, parser)



# -*- coding: utf-8 -*-

# @Time    : 2019/11/12 23:36
# @Software: PyCharm
# @License: BSD 3-Clause
"""
Notes:
    Notes:This file is part of DEAP.
    could be quick generate class with attribute and staticmethod method.
    other methods are not suit for this.
"""
import array
import copy
import warnings
from functools import wraps

import numpy

class_replacers = {}


class _NumpyArray(numpy.ndarray):
    def __deepcopy__(self, memo):
        """Overrides the deepcopy from numpy.ndarray that does not copy
        the object's attributes. This one will deepcopy the array and its
        :attr:`__dict__` attribute.
        """
        copy_ = numpy.ndarray.copy(self)
        copy_.__dict__.update(copy.deepcopy(self.__dict__, memo))
        return copy_

    @staticmethod
    def __new__(cls, iterable):
        """Creates a new instance of a numpy.ndarray from a function call.
        Adds the possibility to instanciate from an iterable."""
        return numpy.array(list(iterable)).view(cls)

    def __setstate__(self, state):
        self.__dict__.update(state)

    def __reduce__(self):
        return (self.__class__, (list(self),), self.__dict__)


class_replacers[numpy.ndarray] = _NumpyArray


class _Array(array.array):
    @staticmethod
    def __new__(cls, seq=(), **kwargs):
        return super(_Array, cls).__new__(cls, cls.typecode, seq)

    def __deepcopy__(self, memo):
        """Overrides the deepcopy from array.array that does not copy
        the object's attributes and class type.
        """
        cls = self.__class__
        copy_ = cls.__new__(cls, self)
        memo[id(self)] = copy_
        copy_.__dict__.update(copy.deepcopy(self.__dict__, memo))
        return copy_

    def __reduce__(self):
        return (self.__class__, (list(self),), self.__dict__)


#
class_replacers[array.array] = _Array


def expand_partial(func):
    """expand the first paramter is self,and with no use. for create"""

    @wraps(func)
    def wrapper(_, *args, **kwargs):
        result = func(*args, **kwargs)
        return result

    return wrapper


def create(name, base, **kwargs):
    """Creates a new class named *name* inheriting from *base* in the
    :mod:`~deap.creator` module. The new class can have attributes defined by
    the subsequent keyword arguments passed to the function create. If the
    argument is a class (without the parenthesis), the __init__ function is
    called in the initialization of an instance of the new object and the
    returned instance is added as an attribute of the class' instance.
    Otherwise, if the argument is not a class, (for example an :class:`int`),
    it is added as a "static" attribute of the class.

    :param name: The name of the class to create.
    :param base: A base class from which to inherit.
    :param kwargs: One or more attributes or function to add on instantiation of this
                      class, optional.

    The following is used to create a class :class:`Foo` inheriting from the
    standard :class:`list` and having an attribute :attr:`bar` being an empty
    dictionary and a static attribute :attr:`spam` initialized to 1. ::
    if add function, the first of argr should left to self.

        def func(*x):
            returns x

        create("Foo", list, bar=dict, spam=1, func=func)

    This above line is exactly the same as defining in the :mod:`creator`
    module something like the following. ::

        class Foo(list):
            spam = 1

            def __init__(spath):
                self.bar = dict()

            def func(self,*x)
                return x

    The :ref:`creating-types` tutorial gives more examples of the creator
    usage.

    .. warning::

       If your are inheriting from :class:`numpy.ndarray` see the
       :doc:`tutorials/advanced/numpy` tutorial and the
       :doc:`/examples/ga_onemax_numpy` example.

    """

    if name in globals():
        warnings.warn("A class named '{0}' has already been created and it "
                      "will be overwritten. Consider deleting previous "
                      "creation of that class or rename it.".format(name),
                      RuntimeWarning)

    dict_inst = {}
    dict_cls = {}
    for obj_name, obj in kwargs.items():
        if isinstance(obj, type):
            dict_inst[obj_name] = obj
        elif callable(obj):
            dict_cls[obj_name] = expand_partial(obj)
        else:
            dict_cls[obj_name] = obj

    # Check if the base class has to be replaced
    if base in class_replacers:
        base = class_replacers[base]

    # A DeprecationWarning is raised when the object inherits from the
    # class "object" which leave the option of passing arguments, but
    # raise a warning stating that it will eventually stop permitting
    # this option. Usually this happens when the base class does not
    # override the __init__ method from object.
    def initType(self, *args, **kargs0):
        """Replace the __init__ function of the new type, in order to
        add attributes that were defined with **kargs to the instance.
        """
        for obj_name0, obj0 in dict_inst.items():
            setattr(self, obj_name0, obj0())
        if base.__init__ is not object.__init__:
            base.__init__(self, *args, **kargs0)

    objtype = type(str(name), (base,), dict_cls)
    objtype.__init__ = initType
    # globals()[name] = objtype
    return objtype


if __name__ == "__main__":
    import numpy as np


    def ff(x, y):
        print(y * 1000)
        return x


    Foo = create("Foo", np.ndarray, lenn=dict, spam=1, fu=ff)
    foo = Foo([1, 2, 3])
    a = foo.fu(2, 4)



"""This is one reserved script, Copy it to your module for use."""
import imp
import os
import platform
from pathlib import Path

import torch
import torch.utils.cpp_extension


def import_module_from_library(module_name, path, is_python_module):
    file, path, description = imp.find_module(module_name, [path])
    # Close the .so file after load.
    with file:
        if is_python_module:
            return imp.load_module(module_name, file, path, description)
        else:
            torch.ops.load_library(path)


def inline_c_torch(module_name="segment_method", source_cpp="inst.cpp", suffix="so", temps="temps",
                   functions=("funcs",)):
    """
    torch.utils.cpp_extension.load, just jump build if exist.

    Parameters
    ----------
    source_cpp:str
        cpp file name.
    temps: str
        Add one temps dir and copy all the file to this disk to escape pollution.
    module_name:str
        name of module
    suffix:str
        module file type.
    functions:str
        name of function in cpp.
    """
    name = module_name

    if platform.system() == "Windows":
        ext = "dll"
    else:
        ext = "so"

    if suffix:
        ext = suffix

    name_dir = temps + name
    MODULE_DIR = Path(source_cpp).parent.absolute()
    MODULE_DIR_NAME_DIR = MODULE_DIR / name_dir

    if os.path.isdir(MODULE_DIR_NAME_DIR) and os.path.isfile(MODULE_DIR_NAME_DIR / "{}.{}".format(name, ext)):
        mod = import_module_from_library(name, MODULE_DIR_NAME_DIR, True)

    else:
        if not os.path.isdir(MODULE_DIR_NAME_DIR):
            os.mkdir(MODULE_DIR_NAME_DIR)

        with open(source_cpp, "a+") as f:
            strs = f.readlines()
            strs = strs[-20:] if len(strs) > 20 else strs
            if any([True if 'PYBIND11_MODULE' in i else False for i in strs]):
                pass
            else:
                if functions is not None:
                    module_def = [" ", ]
                    module_def.append('PYBIND11_MODULE(TORCH_EXTENSION_NAME, m) {')
                    if isinstance(functions, str):
                        functions = [functions]
                    if isinstance(functions, list):
                        # Make the function docstring the same as the function name.
                        functions = dict((f, f) for f in functions)
                    elif not isinstance(functions, dict):
                        raise ValueError(
                            "Expected 'functions' to be a list or dict, but was {}".format(
                                type(functions)))
                    for function_name, docstring in functions.items():
                        module_def.append('m.def("{0}", {0}, "{1}");'.format(function_name, docstring))
                    module_def.append('}')

                    f.writelines(module_def)

        mod = torch.utils.cpp_extension.load(
            name=name,
            sources=[source_cpp, ],
            verbose=True,
            build_directory=MODULE_DIR_NAME_DIR,
            is_python_module=True,
        )

    return mod


if __name__ == '__main__':
    a = inline_c_torch(module_name="segment_method", source_cpp="../inst/speedup_torch.cpp",
                       functions=["d_sigmoid"])
    # inputs = torch.tensor([1.2, 3, 4, 5, 6, 0.7, 9], requires_grad=True, device="cpu")
    # node_bond_idx = [torch.tensor([1, 2, 3]), torch.tensor([4, 5, 6])]
    # b = mod.merge_idx(inputs, node_bond_idx, "mean")



"""This is one reserved script, Copy it to your module for use."""
import imp
import os
import platform
from pathlib import Path

import torch
import torch.utils.cpp_extension


def import_module_from_library(module_name, path, is_python_module):
    file, path, description = imp.find_module(module_name, [path])
    # Close the .so file after load.
    with file:
        if is_python_module:
            return imp.load_module(module_name, file, path, description)
        else:
            torch.ops.load_library(path)


source = """
#include <torch/extension.h>
#include <iostream>

torch::Tensor d_sigmoid(torch::Tensor z) {
  auto s = torch::sigmoid(z);
  return (1 - s) * s;
}
"""


def inline_c_torch(source_cpp=source, module_name="segment_method", temps="temps",
                   suffix="so", functions=('d_sigmoid',)):
    """
    torch.utils.cpp_extension.load_inline, just jump build if exist.

    Parameters
    ----------
    source_cpp:str
        all the cpp source text but not file name.
    temps: str
        Add one temps dir and copy all the file to this disk to escape pollution.
    module_name:str
        name of module
    suffix:str
        module file type.
    functions:str
        name of function in cpp.
    """
    name = module_name

    if platform.system() == "Windows":
        ext = "dll"
    else:
        ext = "so"

    if suffix:
        ext = suffix

    name_dir = temps + name
    MODULE_DIR = Path(__file__).parent.absolute()
    MODULE_DIR_NAME_DIR = MODULE_DIR / name_dir

    if os.path.isdir(MODULE_DIR_NAME_DIR) and os.path.isfile(MODULE_DIR_NAME_DIR / "{}.{}".format(name, ext)):
        mod = import_module_from_library(name, MODULE_DIR_NAME_DIR, True)

    else:
        if not os.path.isdir(MODULE_DIR_NAME_DIR):
            os.mkdir(MODULE_DIR_NAME_DIR)
        mod = torch.utils.cpp_extension.load_inline(
            name=name,
            cpp_sources=source_cpp,
            verbose=True,
            build_directory=MODULE_DIR_NAME_DIR,
            is_python_module=True,
            functions=functions
        )
    return mod


if __name__ == '__main__':
    inputs = torch.tensor([1.2, 3, 4, 5, 6, 0.7, 9], requires_grad=True, device="cpu")
    node_bond_idx = [torch.tensor([1, 2, 3]), torch.tensor([4, 5, 6])]
    # b = mod.merge_idx(inputs, node_bond_idx, "mean")



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-

# @Time   : 2019/7/29 19:44
# @Author : Administrator
# @Software: PyCharm
# @License: BSD 3-Clause

"""
Notes:
    Some tools for characterization
"""
import inspect
import multiprocessing
import numbers
import os
import random
import re
import time
from collections.abc import Iterable
from functools import partial, wraps
from itertools import chain
from sys import getsizeof

import numpy as np
from joblib import Parallel, delayed, effective_n_jobs
from tqdm import tqdm


def time_this_function(func):
    """
    Time the function.
    use as a decorator.

    Examples
    ---------
    ::
        @time_this_function

        def func(x):
            return x
        a= func(1)

    Parameters
    ----------
    func: Callable
        function

    Returns
    -------
    result
        function results
    """

    @wraps(func)
    def wrapper(*args, **kwargs):
        start = time.time()
        result = func(*args, **kwargs)
        end = time.time()
        print(func.__name__, "time", end - start)
        return result

    return wrapper


def check_random_state(seed):
    """
    Turn seed into a random.RandomState instance.
    if use the numpy random, please user the check_random_state in sklearn.

    Parameters
    ----------
    seed: None,int or RandomState
        If seed is None, return the RandomState singleton used by random.
        If seed is an int, return a new RandomState instance seeded with seed.
        If seed is already a RandomState instance, return it.
        Otherwise raise ValueError.

    Returns
    -------
    random.Random
        RandomState object
    """

    if seed is None or seed is random.random:
        return random.Random()
    if isinstance(seed, (numbers.Integral, np.integer)):
        return random.Random(seed)
    if isinstance(seed, random.Random):
        return seed
    raise ValueError('%r cannot be used to seed a seed'
                     ' instance' % seed)


def parallelize(n_jobs, func, iterable, respective=False, tq=True, batch_size='auto', store=None, mode="j",
                parallel_para_dict=None, respective_kwargs=False,
                **kwargs):
    """

    Parallelize the function for iterable.

    Examples
    ----------
    >>> def func(x):
    >>>     return x**2
    >>> result = parallelize(n_jobs=2,func=func,iterable=[1,2,3,4,5])
    [1,2,3,4,5]

    Note:
        For "large" calculated function, with small return for each function.

        def func:
            'large' calculation.
            ...
            ...
            ...
            return int

    make sure in if __name__ == "__main__":

    Parameters
    ----------
    parallel_para_dict:dict
        Parameters passed to joblib.Parallel
    batch_size:str,int

    respective:bool
        Import the parameters respectively or as a whole for each one of iterable object.
    respective_kwargs:
        the respective parameters contains kwargs or not. only for mode=="j" and respective=True
        >>> for iter_i, kw in tqdm(iterable)):
        >>>    func(*iter_i,**kw)

    tq:bool
         View Progress or not
    n_jobs:int
        cpu numbers. n_jobs is the number of workers requested by the callers. Passing n_jobs=-1
    means requesting all available workers for instance matching the number of CPU cores on the worker host(s).
    func:
        function to calculate
    iterable:
        iterable object
    kwargs:
        kwargs for function
    store:
        Not been used.
        store or not, if store, the result would be store to disk and return nothing.
    mode:
        "j":"joblib" or "m":"multiprocessing"
        m for very big data and small loop.

    Returns
    -------
    results
        function results

    """
    if respective_kwargs:
        assert mode == "j" and respective is True

    if parallel_para_dict is None:
        parallel_para_dict = {}
    _ = store
    func = partial(func, **kwargs)

    if mode == "m" and n_jobs != 1:
        pool = multiprocessing.Pool(processes=n_jobs)
        if tq:
            iterable = list(iterable)
            result_list_tqdm = [result for result in tqdm(pool.imap(func=func, iterable=iterable),
                                                          total=len(iterable))]
            pool.close()
        else:
            result_list_tqdm = [result for result in pool.imap(func=func, iterable=iterable)]
            pool.close()
        return result_list_tqdm

    if effective_n_jobs(n_jobs) == 1:
        parallel, func = list, func
    else:
        parallel = Parallel(n_jobs=n_jobs, batch_size=batch_size, **parallel_para_dict)
        func = delayed(func)

    if tq:
        if respective:
            if respective_kwargs:
                # return parallel(func(*iter_i, **kw) if isinstance(iter_i, Iterable) else func(iter_i, **kw) for iter_i, kw in tqdm(iterable))
                return parallel(func(*iter_i, **kw) for *iter_i, kw in tqdm(iterable))
            else:
                return parallel(func(*iter_i) for iter_i in tqdm(iterable))
        else:
            return parallel(func(iter_i) for iter_i in tqdm(iterable))
    else:
        if respective:
            if respective_kwargs:
                # return parallel(func(*iter_i, **kw) if isinstance(iter_i, Iterable) else func(iter_i, **kw)  for iter_i, kw in iterable)
                return parallel(func(*iter_i, **kw) for *iter_i, kw in iterable)
            else:
                return parallel(func(*iter_i) for iter_i in iterable)
        else:
            return parallel(func(iter_i) for iter_i in iterable)


def batch_parallelize(n_jobs, func, iterable, respective=False, tq=True, batch_size: int = 1000, store=None, mode="j",
                      parallel_para_dict: dict = None, respective_kwargs=False,
                      **kwargs):
    """
    Parallelize the function for iterable.

    The iterable would be batched into batch_size  for less resources transmission.

    Examples
    ----------
    >>> def func(x):
    >>>     return x**2
    >>> result = parallelize(n_jobs=2,func=func,iterable=[1,2,3,4,5])

    [1,2,3,4,5]

    Note:
    For "small" calculated function, with small return for each function.

    def func:
        'small' calculation.
        ...
        return int

    make sure in if __name__ == "__main__":

    Parameters
    ----------

    respective_kwargs:
        the respective parameters contains kwargs or not. only for mode=="j" and respective=True.
        the first iterable i is tuple and second kw is dict.
        >>> for iter_i, kw in tqdm(iterable)):
        >>>    func(*iter_i,**kw)

    parallel_para_dict:dict
        Parameters passed to joblib.Parallel
    batch_size:int
        For small data and very big loop.with model "m"
    respective:bool
        Import the parameters respectively or as a whole
    tq:bool
         View Progress or not
    n_jobs:int
        cpu numbers. n_jobs is the number of workers requested by the callers. Passing n_jobs=-1
    means requesting all available workers for instance matching the number of CPU cores on the worker host(s).
    func:
        function to calculate
    iterable:
        iterable object
    mode:
        "j":"joblib" or "m":"multiprocessing"
        m for very big data and small loop.
    kwargs:
        kwargs for function
    store:bool,None
        Not used, store or not, if store, the result would be store to disk and return nothing.

    Returns
    -------
    results
        function results

    """
    if respective_kwargs:
        assert mode == "j" and respective is True

    if parallel_para_dict is None:
        parallel_para_dict = {}

    if effective_n_jobs(n_jobs) == 1:
        return parallelize(n_jobs, func, iterable, respective, tq, batch_size, store,
                           respective_kwargs=respective_kwargs,
                           **kwargs, **parallel_para_dict)

    func = partial(func, **kwargs)

    def func_batch_re(iterablei):
        if respective_kwargs:

            return [func(*i, **kw) for *i, kw in list(iterablei)]
        else:
            return [func(*i) for i in list(iterablei)]

    def func_batch_nre(iterablei):
        return [func(i) for i in list(iterablei)]

    iterable = list(iterable)
    batch = len(iterable) // batch_size + 1
    iterables = np.array_split(iterable, batch)

    parallel = Parallel(n_jobs=n_jobs, batch_size=batch_size, **parallel_para_dict)

    if mode == "m":
        # no tq
        global func_batch_nree

        def func_batch_nree(iterablei):
            return [func(i) for i in list(iterablei)]

        pool = multiprocessing.Pool(processes=n_jobs)
        if tq:
            rett = [result for result in tqdm(pool.imap(func=func_batch_nree, iterable=iterables),
                                              total=len(iterables))]
            pool.close()
        else:
            rett = [result for result in pool.imap(func=func_batch_nree, iterable=iterables)]
            pool.close()
        ret = []
        [ret.extend(i) for i in rett]
        del func_batch_nree
        return ret

    if respective:
        func_batch = delayed(func_batch_re)
    else:
        func_batch = delayed(func_batch_nre)

    try:
        if tq:
            y = parallel(func_batch(iter_i) for iter_i in tqdm(iterables))
        else:
            y = parallel(func_batch_nre(iter_i) for iter_i in iterables)

        ret = []
        [ret.extend(i) for i in y]
        return ret
    except MemoryError:

        raise MemoryError(
            "The total size of calculation is out of Memory, please try ’store‘ result to disk but return to window")


def parallelize_imap(n_jobs, func, iterable, tq=True):
    """
    Parallelize the function for iterable.

    For very big loop and small data.

    Parameters
    ----------
    func:function
    iterable:List
    n_jobs:int
    tq:bool
    """
    pool = multiprocessing.Pool(processes=n_jobs)
    if tq:
        result_list_tqdm = [result for result in tqdm(pool.imap(func=func, iterable=iterable),
                                                      total=len(iterable))]
        pool.close()
    else:
        result_list_tqdm = [result for result in pool.imap(func=func, iterable=iterable)]
        pool.close()

    return result_list_tqdm


def parallelize_parameter(func, iterable, respective=False, **kwargs):
    """Decrease your output size of your function"""
    import multiprocessing
    maxx = multiprocessing.cpu_count()
    n_jobs = maxx - 2

    batch_size = [5, 10, 25, 50, 100]

    iterable2 = []
    for i, j in enumerate(iterable):
        iterable2.append(j)
        if i > 10:
            break
    iterable2 = [iterable2] * 300
    iterable2 = list(chain(*iterable2))

    t1 = time.time()
    if respective:
        a = list(func(*iter_i) for iter_i in iterable2[:16])
    else:
        a = list(func(iter_i) for iter_i in iterable2[:16])
    t2 = time.time()
    t12 = (t2 - t1) / 16
    size = sum([getsizeof(i) for i in a]) / 16 / 1024
    print("Time of calculation/each:%s," % t12, "each output size:%s Kbytes" % size)

    for batch_sizei in batch_size:
        t3 = time.time()
        parallelize(n_jobs, func, iterable2, respective, batch_size=batch_sizei, **kwargs)
        t4 = time.time()
        t34 = t4 - t3
        size2 = size * batch_sizei
        print("Total time/each:%s," % (t34 / 300), "Return output time/each:%s," % (t34 / 300 - t12),
              "Batch output size:%s Kbytes," % size2, "Batch_size:%s" % batch_sizei)

    print("Choice the batch_size with min Total time.")


def logg(func, printing=True, back=False):
    """
    Get the name of function
    use as a decorator

    Parameters
    ----------
    func:Callable
        function to calculate
    printing:bool
        print or not
    back:bool
        return result or not

    Returns
    -------
    function results
    """

    @wraps(func)
    def wrapper(*args, **kwargs):
        if inspect.isclass(func):
            name = "instance of %s" % func.__name__
            arg_dict = ""
            if printing:
                print(name, arg_dict)
            result = func(*args, **kwargs)
        elif inspect.isfunction(func):
            arg_dict = inspect.getcallargs(func, *args, **kwargs)
            name = func.__name__
            if printing:
                print(name, arg_dict)
            result = func(*args, **kwargs)
        else:
            arg_dict = ""
            name = ""
            result = func(*args, **kwargs)
            pass
        if back:
            return (name, arg_dict), result
        else:
            return result

    return wrapper


def name_to_name(*iters, search, search_which=1, return_which=(1,), two_layer=False):
    """
    search and rank the list.

    Parameters
    ----------
    iters:tuple
        iterable objects to select and sort
    search:Iterable
        the rank basis
    search_which:int
        the (the index of iters) of the rank basis. where to rank
    return_which:tuple of int
        return index in inters
    two_layer:bool
        search is nested with two layer or not

    Returns
    -------
    result:Iterable
        Result of find and sort
    """
    if isinstance(return_which, int):
        return_which = tuple([return_which, ])
    if two_layer:

        results_all = []
        if isinstance(search, Iterable):
            for index_i in search:
                results_all.append(
                    name_to_name(*iters, search=index_i, search_which=search_which,
                                 return_which=return_which, two_layer=False))

            if len(return_which) >= 2:
                return list(zip(*results_all))
            else:
                return results_all
        else:
            raise IndexError("search_name or search should be iterable")

    else:

        zeros = [list(range(len(iters[0])))]

        zeros.extend([list(_) for _ in iters])

        iters = zeros

        zips = list(zip(*iters))

        if isinstance(search, Iterable):

            search_index = [iters[search_which].index(i) for i in search]

            results = [zips[i] for i in search_index]

        else:

            raise IndexError("search_name or search should be iterable")

        res = list(zip(*results))

        if not res:
            return_res = [[] for _ in return_which]
        else:
            return_res = [res[_] for _ in return_which]

        if len(return_which) == 1:
            return_res = return_res[0]
        return return_res


class _TTClass(dict):

    def __init__(self, **kwargs):
        super(_TTClass, self).__init__(**kwargs)

    def _t(self):
        self._r()

    def _p(self):
        a = np.array(list(self.keys()))
        b = np.array(list(self.values()))
        a0 = np.delete(a, 0)
        a = np.delete(a, -1)
        b0 = np.delete(b, 0)
        b = np.delete(b, -1)
        ti = b0 - b

        def func(x, y):
            return "{}-{}:".format(x, y)

        ufunc = np.frompyfunc(func, 2, 1)
        ni = ufunc(a0, a)
        re = np.vstack((ni, ti))
        re = re.T
        print(re)
        self.clear()

    def _r(self, name=None):
        ti = time.time()
        if name is None:
            n = len(self)
            name = "t%s" % n
        self[name] = ti

    def subs(self, name1, name2):
        if name2 in self and name2 in self:
            return self[name1] - self[name2]
        else:
            raise NameError("There is no name:{} or {}".format(name1, name2))

    # def tp(self, name=None):
    #     ti = time.time()
    #     n = len(self)
    #     if name is None:
    #         name = "tp"
    #     if n >= 1:
    #         name0 = list(self.keys())[-1]
    #         t0 = list(self.values())[-1]
    #         print("-".join((name,name0)), ti - t0)
    #     else:
    #         raise UserWarning("The .tp only used after .t, or .r")


class TTClass(_TTClass):
    """
    quick time.
    use tt object.

    Examples:
    -----------
    >>> tt.t
    >>> a=4
    >>> tt.t
    >>> tt.p
    [['t1-t0:' ****]]

    """

    def __init__(self, **kwargs):
        super(_TTClass, self).__init__(**kwargs)

    def __getattribute__(self, item):
        if item == "t":
            _TTClass._t(self)

        elif item == "p":
            _TTClass._p(self)

        elif item[-1] in "0123456789":
            _TTClass._r(self, name=item)

        else:
            return _TTClass.__getattribute__(self, item)


def def_pwd(path=None, change=True, verbose=False):
    """try of get and define work path."""
    if path is None:
        path = os.getcwd()
        pwd = path
    if os.path.exists(path):
        path = os.path.abspath(path)
        if change:
            os.chdir(path)
        pwd = os.getcwd()
    else:
        os.makedirs(path)
        path = os.path.abspath(path)
        if change:
            os.chdir(path)
        pwd = os.getcwd()
    if verbose:
        print("work path:", pwd)
        print("checked path:", path)
    locals()[pwd] = pwd
    return path


def get_name_without_suffix(module_name):
    """Get the name without suffix."""
    if "-" in module_name:
        print("'-' in '{}' is replaced by '_'".format(module_name))
    module_name = module_name.replace("-", "_")

    module_fc = re.compile(r"\W")
    module_fc = module_fc.findall(module_name)
    if module_fc[0] == "." and len(module_fc) == 1:
        module_name = module_name.split(".")[0]
    else:
        module_fc.remove(".")
        raise NameError("The string {} in module_name is special character.".format(module_fc))
    print("Confirm the model name: {}".format(module_name))
    return module_name


def cmd_sys(d,cmd):
    """run linux cmd"""
    old = os.getcwd()
    os.chdir(d)
    os.system(cmd)
    os.chdir(old)


def cmd_popen(d,cmd):
    """run linux cmd and return result."""
    old = os.getcwd()
    os.chdir(d)
    res = os.popen(cmd).readlines()
    os.chdir(old)
    return res


tt = TTClass()

if __name__ == "__main__":
    # def func(n, _=None):
    #     # time.sleep(0.0001)
    #     s = np.random.random((100, 50))
    #     return s
    #
    #
    # iterable = np.arange(10000)
    # iterable2 = np.arange(20000)
    # tt.t
    # s = parallelize(1, func, zip(iterable, iterable), respective=True, tq=True, batch_size=1000)
    # tt.t
    # s = parallelize(1, func, iterable, respective=False, tq=True, batch_size=1000)
    # tt.t
    # s = batch_parallelize(1, func, zip(iterable, iterable), respective=True, tq=True, batch_size=1000, )
    # tt.t
    # s = batch_parallelize(1, func, iterable, respective=False, tq=True, batch_size=1000, )
    #
    # tt.t
    # s = parallelize(1, func, list(zip(iterable, iterable)), respective=True, tq=True, batch_size=1000, mode="m")
    # tt.t
    # s = parallelize(1, func, iterable, respective=False, tq=True, batch_size=1000, mode="m")
    # tt.t
    # s = batch_parallelize(1, func, zip(iterable, iterable), respective=True, tq=True, batch_size=1000,
    #                       mode="m")
    # tt.t
    # s = batch_parallelize(1, func, iterable, respective=False, tq=True, batch_size=1000, mode="m")
    # tt.t
    # tt.p

    def func2(n, j, dc=None, dp=None):
        # time.sleep(0.0001)
        assert isinstance(dc, np.int64)
        assert isinstance(dp, np.int64)
        s = np.random.random((100, 50))
        return s


    iterable = np.arange(10000)
    kw = [{"dc": iterable[i], "dp": iterable[i]} for i in range(len(iterable))]
    iterables = zip(iterable, iterable, kw)
    # iterables = zip(iterable,kw)
    tt.t
    s = parallelize(4, func2, iterables, respective=True, tq=True, batch_size=1000, mode="j", respective_kwargs=True)
    tt.t
    tt.p

    tt.t
    s = batch_parallelize(4, func2, iterables, respective=True, tq=True, batch_size=1000, mode="j",
                          respective_kwargs=True)
    tt.t
    tt.p
